@@ -55,10 +55,10 @@ class ligand:
                 hopped += 1
                 new_active_set = list()
                 for this_atom in active_set:
-                    this_atoms_neighbors = self.master_mol.getBondedAtoms(this_atom)
+                    this_atoms_neighbors = self.master_mol.getBondedAtomsSmart(this_atom)
                     for bound_atoms in this_atoms_neighbors:
                         if (bound_atoms in self.index_list) and (bound_atoms not in added_list):
-                            self.trunc_mol.addAtom(self.master_mol.getAtom(bound_atoms))
+                            self.trunc_mol.addAtom(self.master_mol.getAtomSmart(bound_atoms))
                             added_list.append(bound_atoms)
                     [new_active_set.append(element) for element in this_atoms_neighbors]
                 active_set = new_active_set
@@ -72,14 +72,15 @@ class ligand:
 #  @return ligcons List of ligand connection indices (in mol)
 
 
-def ligand_breakdown(mol, flag_loose=False):
+def ligand_breakdown(mol, flag_loose=False, BondedOct=False):
     # this function takes an octahedral
     # complex and returns ligands
     metal_index = mol.findMetal()[0]
-    bondedatoms = mol.getBondedAtomsOct(metal_index, CN=6, debug=False, flag_loose=flag_loose)
-    print('!!!!flagloose', flag_loose)
-    # bondedatoms = mol.getBondedAtomsSmart(metal_index)
-    print('!!!!!boundatoms', bondedatoms)
+    if BondedOct:
+        bondedatoms = mol.getBondedAtomsOct(metal_index, CN=6, debug=False, flag_loose=flag_loose)
+    else:
+        bondedatoms = mol.getBondedAtomsSmart(metal_index)
+    # print('!!!!!boundatoms', bondedatoms)
     #	print('from get oct' + str(bondedatoms))
     #	print('***\n')
     bonded_atom_symbols = [mol.getAtom(i).symbol() for i in bondedatoms]
@@ -225,7 +226,7 @@ def ligand_assign(mol, liglist, ligdents, ligcons, loud=False, name=False):
         allowed = range(0, 6)
         not_eq = list()
         for j, built_ligs in enumerate(built_ligand_list):
-            this_z = built_ligs.mol.centermass()[2]
+            this_z = sum([mol.getAtom(ii).coords()[2] for ii in ligcons[j]])/len(ligcons[j])
             if this_z < minz:
                 minz = this_z
                 bot_lig = j
