@@ -82,6 +82,9 @@ class mol3D:
         self.globs = globalvars()
         ## Holder for molecular graph
         self.graph = []
+        self.xyzfile = 'undef'
+        self.updated = False
+        self.needsconformer = False
 
     ## Add atom to molecule
     #
@@ -214,27 +217,34 @@ class mol3D:
     #
     #  Required for performing openbabel operations on a molecule, such as FF optimizations.
     #  @param self The object pointer
-    #  @param force_clean bool force no bond info retentsion 
-    def convert2OBMol(self, force_clean=False):
+    #  @param force_clean bool force no bond info retention 
+    #  @param ignoreX bool skip "X" atoms in mol3D conversion 
+    def convert2OBMol(self, force_clean=False,ignoreX=False):
 
         # get BO matrix if exits:
         repop = False
 
-        if self.OBMol and not force_clean:
+        if not (self.OBMol== False) and not force_clean:
             BO_mat = self.populateBOMatrix()
-            repop = True 
-        # write temp xyz
-        self.writexyz('tempr.xyz',symbsonly=True)
+
+            repop = True
+            # write temp xyz
+        fd, tempf = tempfile.mkstemp(suffix=".xyz")       
+        os.close(fd)
+        #self.writexyz('tempr.xyz', symbsonly=True)
+        self.writexyz(tempf, symbsonly=True,ignoreX=ignoreX)
 
         obConversion = openbabel.OBConversion()
         obConversion.SetInFormat("xyz")
-
+        
         OBMol = openbabel.OBMol()
-        obConversion.ReadFile(OBMol, 'tempr.xyz')
+        obConversion.ReadFile(OBMol, tempf)
+        
         self.OBMol = []
         self.OBMol = OBMol
+        
 
-        os.remove('tempr.xyz')
+        os.remove(tempf)
 
         if repop and not force_clean:
             self.cleanBonds()
@@ -242,13 +252,14 @@ class mol3D:
                 for j in range(0, self.natoms):
                     if BO_mat[i][j] > 0:
                         self.OBMol.AddBond(i + 1, j + 1, int(BO_mat[i][j]))
-
+        
     ## Combines two molecules
     #
     #  Each atom in the second molecule is appended to the first while preserving orders.
     #  @param self The object pointer
     #  @param mol mol3D containing molecule to be added
     #  @param list of tuples (ind1,ind2,order) bonds to add (optional)
+    #  @param dirty bool set to true for straight addition, not bond safe
     #  @return mol3D contaning combined molecule
     # def combine(self,mol):
     #    cmol = self
@@ -259,44 +270,48 @@ class mol3D:
     #    cmol.graph = []
     #    return cmol
 
-    def combine(self, mol, bond_to_add=[]):
-        # BondSafe
+    def combine(self, mol, bond_to_add=[],dirty=False):
         cmol = self
-        cmol.convert2OBMol()
-        mol.convert2OBMol()
-        n_one = cmol.natoms
-        n_two = mol.natoms
-        n_tot = n_one + n_two
-        # allocate
-        jointBOMat = np.zeros([n_tot, n_tot])
-        # get individual mats 
-        con_mat_one = cmol.populateBOMatrix()
-        con_mat_two = mol.populateBOMatrix()
-        # combine mats
-        for i in range(0, n_one):
-            for j in range(0, n_one):
-                jointBOMat[i][j] = con_mat_one[i][j]
-        for i in range(0, n_two):
-            for j in range(0, n_two):
-                jointBOMat[i + n_one][j + n_one] = con_mat_two[i][j]
-        # optional add additional bond(s)
-        if bond_to_add:
-            for bond_tuples in bond_to_add:
-                print('adding bond ' + str(bond_tuples))
-                jointBOMat[bond_tuples[0], bond_tuples[1]] = bond_tuples[2]
-                jointBOMat[bond_tuples[1], bond_tuples[0]] = bond_tuples[2]
+        
+        if not dirty:
+            # BondSafe
+            cmol.convert2OBMol(force_clean=False,ignoreX=True)            
+            mol.convert2OBMol(force_clean=False,ignoreX=True)
+
+            n_one = cmol.natoms
+            n_two = mol.natoms
+            n_tot = n_one + n_two
+            # allocate
+            jointBOMat = np.zeros([n_tot, n_tot])
+            # get individual mats 
+            con_mat_one = cmol.populateBOMatrix()
+            con_mat_two = mol.populateBOMatrix()
+            # combine mats
+            for i in range(0, n_one):
+                for j in range(0, n_one):
+                    jointBOMat[i][j] = con_mat_one[i][j]
+            for i in range(0, n_two):
+                for j in range(0, n_two):
+                    jointBOMat[i + n_one][j + n_one] = con_mat_two[i][j]
+            # optional add additional bond(s)
+            if bond_to_add:
+                for bond_tuples in bond_to_add:
+                    print('adding bond ' + str(bond_tuples))
+                    jointBOMat[bond_tuples[0], bond_tuples[1]] = bond_tuples[2]
+                    jointBOMat[bond_tuples[1], bond_tuples[0]] = bond_tuples[2]
         # add mol3Ds
         for atom in mol.atoms:
             cmol.addAtom(atom)
-        cmol.convert2OBMol()
-        # clean all bonds
-        cmol.cleanBonds()
-        # restore bond info
-        for i in range(0, n_tot):
-            for j in range(0, n_tot):
-                if jointBOMat[i][j] > 0:
-                    cmol.OBMol.AddBond(i + 1, j + 1, int(jointBOMat[i][j]))
-        # reset graph
+        if not dirty:
+            cmol.convert2OBMol(ignoreX=True)
+            # clean all bonds
+            cmol.cleanBonds()
+            # restore bond info
+            for i in range(0, n_tot):
+                for j in range(0, n_tot):
+                    if jointBOMat[i][j] > 0:
+                        cmol.OBMol.AddBond(i + 1, j + 1, int(jointBOMat[i][j]))
+            # reset graph
         cmol.graph = []
         return cmol
 
@@ -330,10 +345,10 @@ class mol3D:
     #  WARNING: ONLY USE ON A FRESH INSTANCE OF MOL3D.
     #  @param self The object pointer
     #  @param mol0 mol3D of molecule to be copied
-    def copymol3D(self,mol0):
-		# copy atoms
-        for i,atom0 in enumerate(mol0.atoms):
-            self.addAtom(atom3D(atom0.sym,atom0.coords(),atom0.name))
+    def copymol3D(self, mol0):
+        # copy atoms
+        for i, atom0 in enumerate(mol0.atoms):
+            self.addAtom(atom3D(atom0.sym, atom0.coords(), atom0.name))
             if atom0.frozen:
                 self.getAtom(i).frozen = True
         # copy other attributes
@@ -623,6 +638,13 @@ class mol3D:
             distance_max = 1.15 * (atom.rad + ratom.rad)
             # distance_max = 1.25 * (atom.rad + ratom.rad)
             if atom.ismetal() or ratom.ismetal():
+                dist_allowed = {"C":2.8,"H":2.0,"N":2.8,"P":3.0,"I":3.5,"O":2.8}
+                if atom.symbol() in dist_allowed.keys():
+                    max_pos_distance  = dist_allowed[atom.symbol()]
+                elif ratom.symbol() in dist_allowed.keys():
+                    max_pos_distance  = dist_allowed[ratom.symbol()]
+                else:
+                    max_pos_distance  = 2.9 
                 if debug:
                     print('metal in  cat ' + str(atom.symbol()) + ' and rat ' + str(ratom.symbol()))
                 ## one the atoms is a metal!
@@ -632,7 +654,7 @@ class mol3D:
                 if flag_loose:
                     distance_max = min(3.5, 1.75 * (atom.rad + ratom.rad))
                 else:
-                    distance_max = min(2.9, 1.5 * (atom.rad + ratom.rad))
+                    distance_max = min(max_pos_distance, 1.35 * (atom.rad + ratom.rad))
                 # print('distance max:', distance_max)
                 if debug:
                     print('maximum bonded distance is ' + str(distance_max))
@@ -646,6 +668,13 @@ class mol3D:
                         valid = False
                     if d < distance_max and i != ind and valid:
                         if atom.symbol() == "C":
+                            if debug:
+                                print('\n')
+                                self.printxyz()
+                                print('this atom in is ' + str(i))
+                                print('this atom sym is ' + str(atom.symbol()))
+                                print('this ratom in is ' + str(self.getAtom(i).symbol()))
+                                print('this ratom sym is ' + str(ratom.symbol()))
                             ## in this case, atom might be intruder C!
                             possible_inds = self.getBondedAtomsnotH(ind)  ## bonded to metal
                             if debug:
@@ -654,15 +683,19 @@ class mol3D:
                                 metal_prox = sorted(possible_inds, key=lambda x: self.getDistToMetal(x, ind))
 
                                 allowed_inds = metal_prox[0:CN]
+                                # if
                                 if debug:
                                     print('ind: ' + str(ind))
-                                    print('metal prox:' + str(metal_prox))
-                                    print('trimmed to ' + str(allowed_inds))
+                                    print('metal prox: ' + str(metal_prox))
+                                    print('trimmed to: ' + str(allowed_inds))
                                     print(allowed_inds)
+                                    print('CN is ' + str(CN))
+
                                 if not i in allowed_inds:
                                     valid = False
                                     if debug:
                                         print('bond rejected based on atom: ' + str(i) + ' not in ' + str(allowed_inds))
+                                        sardines
                                 else:
                                     if debug:
                                         print('Ok based on atom')
@@ -705,15 +738,42 @@ class mol3D:
                         print('Error, mol3D could not understand conenctivity in mol')
         return nats
 
+    def update_graph_check(self, oct=True): ####!!!!Works only for octahedral and one-empty site!!!!
+        from molSimplify.Scripts.oct_check_mols import IsOct, IsStructure
+        if not len(self.graph):
+            self.createMolecularGraph(oct=oct)
+        if oct:
+            flag_oct, flag_list, dict_oct_info, catoms_arr = IsOct(file_in=self.xyzfile,
+                                                                   flag_catoms=True)
+        else:
+            flag_oct, flag_list, dict_oct_info, catoms_arr = IsStructure(file_in=self.xyzfile,
+                                                                         flag_catoms=True)
+        self.graph[0, :] = 0
+        self.graph[:, 0] = 0
+        row = np.zeros(self.graph.shape[0])
+        np.put(row, np.array(catoms_arr), np.ones(len(catoms_arr)))
+        # print('!!!add', row)
+        self.graph[0, :] = row
+        col = np.zeros((self.graph.shape[1], 1))
+        np.put(col, catoms_arr, np.ones(self.graph.shape[0]))
+        self.graph[:, 0] = col
+        self.updated = True
+        # print(self.graph[0], self.graph[1])
+
     ## Gets atoms bonded to a specific atom using the molecular graph, or creates it
     #
     #  @param self The object pointer
     #  @param ind Index of reference atom
     #  @param oct Flag to turn on special octahedral complex routines
     #  @return List of indices of bonded atoms
-    def getBondedAtomsSmart(self, ind, oct=True):
+    def getBondedAtomsSmart(self, ind, oct=True, geo_check=False):
         if not len(self.graph):
             self.createMolecularGraph(oct=oct)
+        # if (not self.updated):
+        #     print('---------')
+        #     self.update_graph_check()
+        if geo_check:  # Force check
+            self.update_graph_check()
         return list(np.nonzero(np.ravel(self.graph[ind]))[0])
 
     ## Gets non-H atoms bonded to a specific atom
@@ -770,21 +830,23 @@ class mol3D:
             if d0 < dd:
                 dd = d0
                 atomc = atom.coords()
-        return distance(self.centermass(),atomc)
+        return distance(self.centermass(), atomc)
+
     ## Gets atom that is furthest from the given atom
     #  @param self The object pointer
     #  @param reference index of reference atom
     #  @return farIndex index of furthest atom
-    def getFarAtom(self,reference):
+    def getFarAtom(self, reference):
         referenceCoords = self.getAtom(reference).coords()
         dd = 0.00
         farIndex = reference
-        for ind,atom in enumerate(self.atoms):
-            d0 = distance(atom.coords(),referenceCoords)
+        for ind, atom in enumerate(self.atoms):
+            d0 = distance(atom.coords(), referenceCoords)
             if d0 > dd:
                 dd = d0
                 farIndex = ind
         return farIndex
+
     ## Gets H atoms in molecule
     #  @param self The object pointer
     #  @return List of atom3D objects of H atoms
@@ -919,6 +981,7 @@ class mol3D:
         return idx
 
     ## Initializes OBMol object from a file or SMILES string
+    
     #
     #  Uses the obConversion tool and for files containing 3D coordinates (xyz,mol) and the OBBuilder tool otherwise (smiles).
     #  @param self The object pointer    
@@ -1061,7 +1124,7 @@ class mol3D:
     ## Checks for overlap with another molecule with increased tolerance
     # 
     #  Compares pairwise atom distances to 1*sum of covalent radii
-        #  @param self The object pointer    
+    #  @param self The object pointer
     #  @param mol mol3D of second molecule
     #  @return Flag for overlap
     def overlapcheckh(self, mol):
@@ -1103,9 +1166,11 @@ class mol3D:
     #  @param self The object pointer    
     #  @param filename Filename
     def readfromxyz(self, filename):
+        # print('!!!!', filename)
         globs = globalvars()
-        en_dict = globs.endict()
+        amassdict = globs.amass()
         self.graph = []
+        self.xyzfile = filename
         fname = filename.split('.xyz')[0]
         f = open(fname + '.xyz', 'r')
         s = f.read().splitlines()
@@ -1117,18 +1182,45 @@ class mol3D:
                 lm = re.search(r'\d+$', line_split[0])
                 # if the string ends in digits m will be a Match object, or None otherwise.
                 if lm is not None:
-                    symb = re.sub('\d+','',line_split[0])
+                    symb = re.sub('\d+', '', line_split[0])
                     number = lm.group()
-                  # print('sym and number ' +str(symb) + ' ' + str(number))
+                    # print('sym and number ' +str(symb) + ' ' + str(number))
                     globs = globalvars()
-                    atom = atom3D(symb,[float(line_split[1]),float(line_split[2]),float(line_split[3])],name=line_split[0])
-                elif line_split[0] in en_dict.keys():
-                    atom = atom3D(line_split[0],[float(line_split[1]),float(line_split[2]),float(line_split[3])])
+                    atom = atom3D(symb, [float(line_split[1]), float(line_split[2]), float(line_split[3])],
+                                  name=line_split[0])
+                elif line_split[0] in amassdict.keys():
+                    atom = atom3D(line_split[0], [float(line_split[1]), float(line_split[2]), float(line_split[3])])
                 else:
                     print('cannot find atom type')
                     sys.exit()
                 self.addAtom(atom)
-                
+
+    def readfromtxt(self, txt):
+    # print('!!!!', filename)
+        globs = globalvars()
+        en_dict = globs.endict()
+        self.graph = []
+        for line in txt:
+            line_split = line.split()
+            if len(line_split) == 4 and line_split[0]:
+                # this looks for unique atom IDs in files
+                lm = re.search(r'\d+$', line_split[0])
+                # if the string ends in digits m will be a Match object, or None otherwise.
+                if lm is not None:
+                    symb = re.sub('\d+', '', line_split[0])
+                    # number = lm.group()
+                    # # print('sym and number ' +str(symb) + ' ' + str(number))
+                    # globs = globalvars()
+                    atom = atom3D(symb, [float(line_split[1]), float(line_split[2]), float(line_split[3])],
+                                  name=line_split[0])
+                elif line_split[0] in en_dict.keys():
+                    atom = atom3D(line_split[0], [float(line_split[1]), float(line_split[2]), float(line_split[3])])
+                else:
+                    print('cannot find atom type')
+                    sys.exit()
+                self.addAtom(atom)
+
+
     ## Computes RMSD between two molecules
     # 
     #  Note that this routine does not perform translations or rotations to align molecules.
@@ -1147,13 +1239,38 @@ class mol3D:
             rmsd = 0
             for atom0, atom1 in zip(self.getAtoms(), mol2.getAtoms()):
                 rmsd += (atom0.distance(atom1)) ** 2
-            N_nonH = Nat0- len(self.getHs())
-            if N_nonH:
-                rmsd /= Nat0
-            else:
+            if Nat0 == 0:
                 rmsd = 0
+            else:
+                rmsd /= Nat0
             return sqrt(rmsd)
 
+    ## Computes mean of absolute atom deviations 
+    # 
+    #  Like above, this routine does not perform translations or rotations to align molecules.
+    #  
+    #  Use mol3D objects that do not use hydrogens
+    #
+    #  To do so, use geometry.kabsch().
+    #  @param self The object pointer  
+    #  @param mol2 mol3D of second molecule
+    #  @return sum of absolute deviations of atoms between molecules, NaN if molecules have different numbers of atoms
+    def meanabsdev(self, mol2):
+        Nat0 = self.natoms
+        Nat1 = mol2.natoms
+        if (Nat0 != Nat1):
+            print "ERROR: Absolute atom deviations can be calculated only for molecules with the same number of atoms.."
+            return NaN
+        else:
+            dev = 0
+            for atom0, atom1 in zip(self.getAtoms(), mol2.getAtoms()):
+                dev += abs((atom0.distance(atom1)))
+            if Nat0 == 0:
+                dev = 0
+            else:
+                dev /= Nat0
+            return dev
+
     def maxatomdist(self, mol2):
         Nat0 = self.natoms
         Nat1 = mol2.natoms
@@ -1177,7 +1294,7 @@ class mol3D:
         else:
             rmsd = 0
             for atom0, atom1 in zip(self.getAtoms(), mol2.getAtoms()):
-                if (not atom0.sym =='H') and (not atom1.sym =='H'):
+                if (not atom0.sym == 'H') and (not atom1.sym == 'H'):
                     rmsd += (atom0.distance(atom1)) ** 2
             rmsd /= Nat0
             return sqrt(rmsd)
@@ -1191,7 +1308,7 @@ class mol3D:
             return NaN
         else:
             for atom0, atom1 in zip(self.getAtoms(), mol2.getAtoms()):
-                if (not atom0.sym =='H') and (not atom1.sym =='H'):
+                if (not atom0.sym == 'H') and (not atom1.sym == 'H'):
                     dist = atom0.distance(atom1)
                     if dist > dist_max:
                         dist_max = dist
@@ -1204,7 +1321,7 @@ class mol3D:
     #  @param silence Flag for printing warning
     #  @return Flag for overlap
     #  @return Minimum distance between atoms
-    def sanitycheck(self, silence):
+    def sanitycheck(self, silence=False):
         overlap = False
         mind = 1000
         for ii, atom1 in enumerate(self.atoms):
@@ -1247,15 +1364,21 @@ class mol3D:
     #  To print to stdout instead, use printxyz().
     #  @param self The object pointer   
     #  @param filename Filename  
-    def writexyz(self,filename,symbsonly=False):
-        ss = '' # initialize returning string       
-        ss += str(self.natoms)+"\n"+time.strftime('%m/%d/%Y %H:%M')+", XYZ structure generated by mol3D Class, "+self.globs.PROGRAM+"\n"
+    def writexyz(self, filename, symbsonly=False,ignoreX=False):
+        ss = ''  # initialize returning string
+        natoms = self.natoms
+        if ignoreX:
+            natoms -= sum([1 for i in self.atoms if i.sym == "X"])
+        
+        ss += str(natoms) + "\n" + time.strftime(
+            '%m/%d/%Y %H:%M') + ", XYZ structure generated by mol3D Class, " + self.globs.PROGRAM + "\n"
         for atom in self.atoms:
-            xyz = atom.coords()
-            if symbsonly:
-                ss += "%s \t%f\t%f\t%f\n" % (atom.sym,xyz[0],xyz[1],xyz[2])
-            else:
-                ss += "%s \t%f\t%f\t%f\n" % (atom.name,xyz[0],xyz[1],xyz[2])
+            if not (ignoreX and atom.sym == 'X'):
+                xyz = atom.coords()
+                if symbsonly:
+                    ss += "%s \t%f\t%f\t%f\n" % (atom.sym, xyz[0], xyz[1], xyz[2])
+                else:
+                    ss += "%s \t%f\t%f\t%f\n" % (atom.name, xyz[0], xyz[1], xyz[2])
         fname = filename.split('.xyz')[0]
         f = open(fname + '.xyz', 'w')
         f.write(ss)
