@@ -17,81 +17,7 @@ from molSimplify.Classes.ligand import *
 HF_to_Kcal_mol = 627.503
 
 
-def full_autocorrelation(mol, prop, d, oct=oct):
-    w = construct_property_vector(mol, prop, oct=oct)
-    index_set = range(0, mol.natoms)
-    autocorrelation_vector = numpy.zeros(d + 1)
-    for centers in index_set:
-        autocorrelation_vector += autocorrelation(mol, w, centers, d, oct=oct)
-    return (autocorrelation_vector)
-
-
-def atom_only_autocorrelation(mol, prop, d, atomIdx, oct=True):
-    ## atomIdx must b either a list of indcies
-    ## or a single index
-    w = construct_property_vector(mol, prop, oct=True)
-    autocorrelation_vector = numpy.zeros(d + 1)
-    if hasattr(atomIdx, "__len__"):
-        for elements in atomIdx:
-            autocorrelation_vector += autocorrelation(mol, w, elements, d, oct=oct)
-        autocorrelation_vector = np.divide(autocorrelation_vector, len(atomIdx))
-    else:
-        autocorrelation_vector += autocorrelation(mol, w, atomIdx, d, oct=oct)
-    return (autocorrelation_vector)
-
-
-def metal_only_autocorrelation(mol, prop, d, oct=True):
-    autocorrelation_vector = numpy.zeros(d)
-    try:
-        metal_ind = mol.findMetal()[0]
-        w = construct_property_vector(mol, prop, oct=oct)
-        autocorrelation_vector = autocorrelation(mol, w, metal_ind, d, oct=oct)
-    except:
-        print('Error, no metal found in mol object!')
-        return False
-    return (autocorrelation_vector)
-
-
-def atom_only_deltametric(mol, prop, d, atomIdx, oct=True):
-    ## atomIdx must b either a list of indcies
-    ## or a single index
-    w = construct_property_vector(mol, prop, oct=oct)
-    deltametric_vector = numpy.zeros(d + 1)
-    if hasattr(atomIdx, "__len__"):
-        for elements in atomIdx:
-            deltametric_vector += deltametric(mol, w, elements, d, oct=oct)
-        deltametric_vector = np.divide(deltametric_vector, len(atomIdx))
-    else:
-        deltametric_vector += deltametric(mol, w, atomIdx, d, oct=oct)
-    return (deltametric_vector)
-
-
-def metal_only_deltametric(mol, prop, d, oct=True):
-    deltametric_vector = numpy.zeros(d)
-    try:
-        metal_ind = mol.findMetal()[0]
-        w = construct_property_vector(mol, prop, oct=oct)
-        deltametric_vector = deltametric(mol, w, metal_ind, d, oct=oct)
-    except:
-        print('Error, no metal found in mol object!')
-        return False
-    return (deltametric_vector)
-
-
-def metal_only_layer_density(mol, prop, d, oct=True):
-    density_vector = numpy.zeros(d)
-    try:
-        metal_ind = mol.findMetal()[0]
-        print ('metal_index is: %d' % metal_ind)
-        w = construct_property_vector(mol, prop, oct=oct)
-        density_vector = layer_density_in_3D(mol, w, metal_ind, d, oct=oct)
-    except:
-        print('Error, no metal found in mol object!')
-        return False
-    return density_vector
-
-
-def autocorrelation(mol, prop_vec, orig, d, oct=True):
+def autocorrelation(mol, prop_vec, orig, d, oct=True, catoms=None):
     ## this function returns the autocorrelation
     ## for one atom
     # Inputs:
@@ -128,7 +54,7 @@ def autocorrelation(mol, prop_vec, orig, d, oct=True):
     return (result_vector)
 
 
-def deltametric(mol, prop_vec, orig, d, oct=True):
+def deltametric(mol, prop_vec, orig, d, oct=True, catoms=None):
     ## this function returns the deltametric
     ## over the whole molecule
     # Inputs:
@@ -164,6 +90,148 @@ def deltametric(mol, prop_vec, orig, d, oct=True):
     return (result_vector)
 
 
+def autocorrelation_catoms(mol, prop_vec, orig, d, oct=True, catoms=None):
+    # Calculate the autocorrelation for the orig to certain connecting atoms.
+    result_vector = numpy.zeros(d + 1)
+    hopped = 0
+    active_set = set([orig])
+    historical_set = set()
+    result_vector[hopped] = prop_vec[orig] * prop_vec[orig]
+    #	if oct:
+    #		print('using OCT autocorrelation')
+    #	#else:
+    #		print('NOT using OCT autocorrelation')
+    while hopped < (d):
+
+        hopped += 1
+        new_active_set = set()
+        for this_atom in active_set:
+            ## prepare all atoms attached to this connection
+            # print('called in AC')
+            this_atoms_neighbors = mol.getBondedAtomsSmart(this_atom, oct=oct)
+            # print('--1--:', this_atoms_neighbors)
+            if this_atom == orig and (not catoms == None):
+                this_atoms_neighbors = catoms
+            # print('--2--:', this_atoms_neighbors)
+            for bound_atoms in this_atoms_neighbors:
+                if (bound_atoms not in historical_set) and (bound_atoms not in active_set):
+                    new_active_set.add(bound_atoms)
+        # print('new active set at hop = ' +str(hopped) + ' is ' +str(new_active_set))
+        for inds in new_active_set:
+            result_vector[hopped] += prop_vec[orig] * prop_vec[inds]
+            historical_set.update(active_set)
+        active_set = new_active_set
+    return (result_vector)
+
+
+def deltametric_catoms(mol, prop_vec, orig, d, oct=True, catoms=None):
+    # Calculate the deltametrics for the orig to certain connecting atoms.
+    result_vector = numpy.zeros(d + 1)
+    hopped = 0
+    active_set = set([orig])
+    historical_set = set()
+    result_vector[hopped] = 0.00
+    # metal_idx = mol.findMetal()[0]
+    while hopped < (d):
+        hopped += 1
+        new_active_set = set()
+        for this_atom in active_set:
+            ## prepare all atoms attached to this connection
+            # print('called in DAC')
+            this_atoms_neighbors = mol.getBondedAtomsSmart(this_atom, oct=oct)
+            # print('--1--:', this_atoms_neighbors)
+            if this_atom == orig and (not catoms == None):
+                this_atoms_neighbors = catoms
+            # print('--2--:', this_atoms_neighbors)
+            for bound_atoms in this_atoms_neighbors:
+                if (bound_atoms not in historical_set) and (bound_atoms not in active_set):
+                    new_active_set.add(bound_atoms)
+        # print('new active set at hop = ' +str(hopped) + ' is ' +str(new_active_set))
+        for inds in new_active_set:
+            result_vector[hopped] += prop_vec[orig] - prop_vec[inds]
+            historical_set.update(active_set)
+        active_set = new_active_set
+    return (result_vector)
+
+
+def full_autocorrelation(mol, prop, d, oct=oct):
+    w = construct_property_vector(mol, prop, oct=oct)
+    index_set = range(0, mol.natoms)
+    autocorrelation_vector = numpy.zeros(d + 1)
+    for centers in index_set:
+        autocorrelation_vector += autocorrelation(mol, w, centers, d, oct=oct)
+    return (autocorrelation_vector)
+
+
+def atom_only_autocorrelation(mol, prop, d, atomIdx, oct=True):
+    ## atomIdx must b either a list of indcies
+    ## or a single index
+    w = construct_property_vector(mol, prop, oct=True)
+    autocorrelation_vector = numpy.zeros(d + 1)
+    if hasattr(atomIdx, "__len__"):
+        for elements in atomIdx:
+            autocorrelation_vector += autocorrelation(mol, w, elements, d, oct=oct)
+        autocorrelation_vector = np.divide(autocorrelation_vector, len(atomIdx))
+    else:
+        autocorrelation_vector += autocorrelation(mol, w, atomIdx, d, oct=oct)
+    return (autocorrelation_vector)
+
+
+def metal_only_autocorrelation(mol, prop, d, oct=True, catoms=None,
+                               func=autocorrelation,modifier=False):
+    autocorrelation_vector = numpy.zeros(d)
+    try:
+        metal_ind = mol.findMetal()[0]
+        w = construct_property_vector(mol, prop, oct=oct,modifier=modifier)
+        autocorrelation_vector = func(mol, w, metal_ind, d, oct=oct,
+                                      catoms=catoms)
+    except:
+        print('Error, no metal found in mol object!')
+        return False
+    return (autocorrelation_vector)
+
+
+def atom_only_deltametric(mol, prop, d, atomIdx, oct=True,modifier=False):
+    ## atomIdx must b either a list of indcies
+    ## or a single index
+    w = construct_property_vector(mol, prop, oct=oct,modifier=modifier)
+    deltametric_vector = numpy.zeros(d + 1)
+    if hasattr(atomIdx, "__len__"):
+        for elements in atomIdx:
+            deltametric_vector += deltametric(mol, w, elements, d, oct=oct)
+        deltametric_vector = np.divide(deltametric_vector, len(atomIdx))
+    else:
+        deltametric_vector += deltametric(mol, w, atomIdx, d, oct=oct)
+    return (deltametric_vector)
+
+
+def metal_only_deltametric(mol, prop, d, oct=True, catoms=None,
+                           func=deltametric, modifier=False):
+    deltametric_vector = numpy.zeros(d)
+    try:
+        metal_ind = mol.findMetal()[0]
+        w = construct_property_vector(mol, prop, oct=oct,modifier=modifier)
+        deltametric_vector = func(mol, w, metal_ind, d, oct=oct,
+                                  catoms=catoms)
+    except:
+        print('Error, no metal found in mol object!')
+        return False
+    return (deltametric_vector)
+
+
+def metal_only_layer_density(mol, prop, d, oct=True):
+    density_vector = numpy.zeros(d)
+    try:
+        metal_ind = mol.findMetal()[0]
+        print ('metal_index is: %d' % metal_ind)
+        w = construct_property_vector(mol, prop, oct=oct)
+        density_vector = layer_density_in_3D(mol, w, metal_ind, d, oct=oct)
+    except:
+        print('Error, no metal found in mol object!')
+        return False
+    return density_vector
+
+
 def layer_density_in_3D(mol, prop_vec, orig, d, oct=True):
     ## this function returns the density (prop^3/(d+1)^3)
     ## for one atom
@@ -201,16 +269,18 @@ def layer_density_in_3D(mol, prop_vec, orig, d, oct=True):
     return result_vector
 
 
-def construct_property_vector(mol, prop, oct=True):
+def construct_property_vector(mol, prop, oct=True,modifier = False):
     ## assigns the value of property
     ## for atom i (zero index) in mol
     ## to position i in returned vector
     ## can be used to create weighted
     ## graph representations
-    #	oct - bool, if complex is octahedral, will use better bond checks
-
-    # allowed_strings = ['electronegativity','nulcear_charge','ident','coord']
-    allowed_strings = ['electronegativity', 'nuclear_charge', 'ident', 'topology', 'size', 'vdwrad']
+    ## oct - bool, if complex is octahedral, will use better bond checks
+    ## modifier - dict, used to modify prop vector (e.g. for adding 
+    ##             ONLY used with  ox_nuclear_charge    ox or charge)
+    ##              {"Fe":2, "Co": 3} etc
+    allowed_strings = ['electronegativity', 'nuclear_charge', 'ident', 'topology',
+                        'ox_nuclear_charge', 'size', 'vdwrad']
     ## note that ident just codes every atom as one, this gives
     ## a purely toplogical index. coord gives the number of
     ## connecting atom to attom i (similar to Randic index)
@@ -236,6 +306,17 @@ def construct_property_vector(mol, prop, oct=True):
         for keys in at_keys:
             values = globs.amass()[keys][1]
             prop_dict.update({keys: values})
+    elif prop == 'ox_nuclear_charge':
+        if not modifier:
+            print('Error, must give modifier with ox_nuclear_charge')
+            return False
+        else:
+            at_keys = globs.amass().keys()
+            for keys in at_keys:
+                values = globs.amass()[keys][1]
+                if keys in modifier.keys():
+                    values += float(modifier[keys])
+                prop_dict.update({keys: values})
     elif prop == 'ident':
         at_keys = globs.amass().keys()
         for keys in at_keys:
@@ -263,16 +344,29 @@ def construct_property_vector(mol, prop, oct=True):
     return (w)
 
 
-def find_ligand_autocorrelations_oct(mol, prop, loud, depth, name=False, oct=True):
+def find_ligand_autocorrelations_oct(mol, prop, loud, depth, name=False,
+                                        oct=True,custom_ligand_dict = False):
     ## this function takes a
     ## symmetric (axial == axial,
     ## equatorial == equatorial)
     ## octahedral complex
     ## and returns autocorrelations for
     ## the axial an equatorial ligands
-    liglist, ligdents, ligcons = ligand_breakdown(mol)
-    ax_ligand_list, eq_ligand_list, ax_natoms_list, eq_natoms_list, ax_con_int_list, eq_con_int_list, ax_con_list, eq_con_list, built_ligand_list = ligand_assign(
-        mol, liglist, ligdents, ligcons, loud, name)
+    ## custom_ligand_dict allows the user to skip the breakdown
+    ## in cases where 3D geo is not correct/formed 
+    ## custom_ligand_dict.keys() must be eq_ligands_list, ax_ligand_list
+    ##                                    ax_con_int_list ,eq_con_int_list
+    ## with types: eq/ax_ligand_list list of mol3D
+    ##             eq/ax_con_int_list list of list/tuple of int e.g,  [[1,2] [1,2]]
+    if not custom_ligand_dict:
+        liglist, ligdents, ligcons = ligand_breakdown(mol)
+        ax_ligand_list, eq_ligand_list, ax_natoms_list, eq_natoms_list, ax_con_int_list, eq_con_int_list, ax_con_list, eq_con_list, built_ligand_list = ligand_assign(
+            mol, liglist, ligdents, ligcons, loud, name)
+    else:
+        ax_ligand_list = custom_ligand_dict["ax_ligand_list"]
+        eq_ligand_list = custom_ligand_dict["eq_ligand_list"]
+        ax_con_int_list = custom_ligand_dict["ax_con_int_list"]
+        eq_con_int_list = custom_ligand_dict["eq_con_int_list"]
     ## count ligands
     n_ax = len(ax_ligand_list)
     n_eq = len(eq_ligand_list)
@@ -314,14 +408,24 @@ def find_ligand_autocorrelations_oct(mol, prop, loud, depth, name=False, oct=Tru
     return ax_ligand_ac_full, eq_ligand_ac_full, ax_ligand_ac_con, eq_ligand_ac_con
 
 
-def find_ligand_deltametrics_oct(mol, prop, loud, depth, name=False, oct=True):
+def find_ligand_deltametrics_oct(mol, prop, loud, depth, name=False, oct=True,custom_ligand_dict=False):
+    ## custom_ligand_dict.keys() must be eq_ligands_list, ax_ligand_list
+    ##                                    ax_con_int_list ,eq_con_int_list
+    ## with types: eq/ax_ligand_list list of mol3D
+    ##             eq/ax_con_int_list list of list/tuple of int e.g,  [[1,2] [1,2]]
     ## this function takes a
     ## octahedral complex
     ## and returns deltametrics for
     ## the axial an equatorial ligands
-    liglist, ligdents, ligcons = ligand_breakdown(mol)
-    ax_ligand_list, eq_ligand_list, ax_natoms_list, eq_natoms_list, ax_con_int_list, eq_con_int_list, ax_con_list, eq_con_list, built_ligand_list = ligand_assign(
-        mol, liglist, ligdents, ligcons, loud, name)
+    if not custom_ligand_dict:
+        liglist, ligdents, ligcons = ligand_breakdown(mol)
+        ax_ligand_list, eq_ligand_list, ax_natoms_list, eq_natoms_list, ax_con_int_list, eq_con_int_list, ax_con_list, eq_con_list, built_ligand_list = ligand_assign(
+            mol, liglist, ligdents, ligcons, loud, name)
+    else:
+        ax_ligand_list = custom_ligand_dict["ax_ligand_list"]
+        eq_ligand_list = custom_ligand_dict["eq_ligand_list"]
+        ax_con_int_list = custom_ligand_dict["ax_con_int_list"]
+        eq_con_int_list = custom_ligand_dict["eq_con_int_list"]
     ## count ligands
     n_ax = len(ax_ligand_list)
     n_eq = len(eq_ligand_list)
@@ -346,7 +450,88 @@ def find_ligand_deltametrics_oct(mol, prop, loud, depth, name=False, oct=True):
     return ax_ligand_ac_con, eq_ligand_ac_con
 
 
-def generate_all_ligand_autocorrelations(mol, loud, depth=4, name=False):
+def find_mc_eq_ax_deltametrics_oct(mol, prop, loud, depth, name=False, oct=True,
+                                   func=deltametric_catoms):
+    # For octahedral complexes only.
+    # Calculate mc/ax, mc/eq deltametrics.
+    liglist, ligdents, ligcons = ligand_breakdown(mol)
+    ax_ligand_list, eq_ligand_list, ax_natoms_list, eq_natoms_list, ax_con_int_list, eq_con_int_list, ax_con_list, eq_con_list, built_ligand_list = ligand_assign(
+        mol, liglist, ligdents, ligcons, loud, name)
+    ## shape reduce
+    ax_con_list = [x[0] for x in ax_con_list]
+    eq_con_list = [x[0] for x in eq_con_list]
+    ax_ligand_del_mc = metal_only_deltametric(mol, prop, depth, catoms=ax_con_list, func=func)
+    eq_ligand_del_mc = metal_only_deltametric(mol, prop, depth, catoms=eq_con_list, func=func)
+    ax_ligand_del_mc = np.divide(ax_ligand_del_mc, len(ax_con_list))
+    eq_ligand_del_mc = np.divide(eq_ligand_del_mc, len(eq_con_list))
+    return ax_ligand_del_mc, eq_ligand_del_mc
+
+
+def find_mc_eq_ax_autocorrelation_oct(mol, prop, loud, depth, name=False, oct=True,
+                                      func=autocorrelation_catoms):
+    # For octahedral complexes only.
+    # Calculate mc/ax, mc/eq deltametrics.
+    liglist, ligdents, ligcons = ligand_breakdown(mol)
+    ax_ligand_list, eq_ligand_list, ax_natoms_list, eq_natoms_list, ax_con_int_list, eq_con_int_list, ax_con_list, eq_con_list, built_ligand_list = ligand_assign(
+        mol, liglist, ligdents, ligcons, loud, name)
+    ## shape reduce
+    ax_con_list = [x[0] for x in ax_con_list]
+    eq_con_list = [x[0] for x in eq_con_list]
+    ax_ligand_ac_mc = metal_only_autocorrelation(mol, prop, depth, catoms=ax_con_list, func=func)
+    eq_ligand_ac_mc = metal_only_autocorrelation(mol, prop, depth, catoms=eq_con_list, func=func)
+    ax_ligand_ac_mc = np.divide(ax_ligand_ac_mc, len(ax_con_list))
+    eq_ligand_ac_mc = np.divide(eq_ligand_ac_mc, len(eq_con_list))
+    return ax_ligand_ac_mc, eq_ligand_ac_mc
+
+
+def generate_mc_eq_ax_deltametrics(mol, loud, depth=4, name=False,
+                                   func=deltametric_catoms):
+    result_ax_mc = list()
+    result_eq_mc = list()
+    colnames = []
+    allowed_strings = ['electronegativity', 'nuclear_charge', 'ident', 'topology', 'size']
+    labels_strings = ['chi', 'Z', 'I', 'T', 'S']
+    
+    for ii, properties in enumerate(allowed_strings):
+        ax_ligand_ac_con, eq_ligand_ac_con = find_mc_eq_ax_deltametrics_oct(mol, properties, loud, depth, name,
+                                                                            func=func)
+        this_colnames = []
+        for i in range(0, depth + 1):
+            this_colnames.append(labels_strings[ii] + '-' + str(i))
+        colnames.append(this_colnames)
+        result_ax_mc.append(ax_ligand_ac_con)
+        result_eq_mc.append(eq_ligand_ac_con)
+    results_dictionary = {'colnames': colnames, 'result_mc_ax_del': result_ax_mc,
+                          'result_mc_eq_del': result_eq_mc}
+    return results_dictionary
+
+
+def generate_mc_eq_ax_autocorrelation(mol, loud, depth=4, name=False,
+                                      func=autocorrelation_catoms):
+    result_ax_mc = list()
+    result_eq_mc = list()
+    colnames = []
+    allowed_strings = ['electronegativity', 'nuclear_charge', 'ident', 'topology', 'size']
+    labels_strings = ['chi', 'Z', 'I', 'T', 'S']
+    for ii, properties in enumerate(allowed_strings):
+        ax_ligand_ac_con, eq_ligand_ac_con = find_mc_eq_ax_autocorrelation_oct(mol, properties, loud, depth, name,
+                                                                               func=func)
+        this_colnames = []
+        for i in range(0, depth + 1):
+            this_colnames.append(labels_strings[ii] + '-' + str(i))
+        colnames.append(this_colnames)
+        result_ax_mc.append(ax_ligand_ac_con)
+        result_eq_mc.append(eq_ligand_ac_con)
+    results_dictionary = {'colnames': colnames, 'result_mc_ax_ac': result_ax_mc,
+                          'result_mc_eq_ac': result_eq_mc}
+    return results_dictionary
+
+
+def generate_all_ligand_autocorrelations(mol, loud, depth=4, name=False, flag_name=False,custom_ligand_dict=False):
+    ## custom_ligand_dict.keys() must be eq_ligands_list, ax_ligand_list
+    ##                                    ax_con_int_list ,eq_con_int_list
+    ## with types: eq/ax_ligand_list list of mol3D
+    ##             eq/ax_con_int_list list of list/tuple of int e.g,  [[1,2] [1,2]]
     result_ax_full = list()
     result_eq_full = list()
     result_ax_con = list()
@@ -357,9 +542,11 @@ def generate_all_ligand_autocorrelations(mol, loud, depth=4, name=False):
     for ii, properties in enumerate(allowed_strings):
         ax_ligand_ac_full, eq_ligand_ac_full, ax_ligand_ac_con, eq_ligand_ac_con = find_ligand_autocorrelations_oct(mol,
                                                                                                                     properties,
-                                                                                                                    loud,
-                                                                                                                    depth,
-                                                                                                                    name)
+                                                                                                                    loud = loud,
+                                                                                                                    depth = depth,
+                                                                                                                    name =name,
+                                                                                                                    oct=True,
+                                                                                                                    custom_ligand_dict= custom_ligand_dict)
         this_colnames = []
         for i in range(0, depth + 1):
             this_colnames.append(labels_strings[ii] + '-' + str(i))
@@ -368,14 +555,21 @@ def generate_all_ligand_autocorrelations(mol, loud, depth=4, name=False):
         result_eq_full.append(eq_ligand_ac_full)
         result_ax_con.append(ax_ligand_ac_con)
         result_eq_con.append(eq_ligand_ac_con)
-    # results_dictionary = {'colnames': colnames, 'result_ax_full_ac': result_ax_full, 'result_eq_full_ac': result_eq_full,
-    #                       'result_ax_con_ac': result_ax_con, 'result_eq_con_ac': result_eq_con}
-    results_dictionary = {'colnames': colnames, 'result_ax_full': result_ax_full, 'result_eq_full': result_eq_full,
-                          'result_ax_con': result_ax_con, 'result_eq_con': result_eq_con}
+    if flag_name:
+        results_dictionary = {'colnames': colnames, 'result_ax_full_ac': result_ax_full,
+                              'result_eq_full_ac': result_eq_full,
+                              'result_ax_con_ac': result_ax_con, 'result_eq_con_ac': result_eq_con}
+    else:
+        results_dictionary = {'colnames': colnames, 'result_ax_full': result_ax_full, 'result_eq_full': result_eq_full,
+                              'result_ax_con': result_ax_con, 'result_eq_con': result_eq_con}
     return results_dictionary
 
 
-def generate_all_ligand_deltametrics(mol, loud, depth=4, name=False):
+def generate_all_ligand_deltametrics(mol, loud, depth=4, name=False, flag_name=False,custom_ligand_dict=False):
+    ## custom_ligand_dict.keys() must be eq_ligands_list, ax_ligand_list
+    ##                                    ax_con_int_list ,eq_con_int_list
+    ## with types: eq/ax_ligand_list list of mol3D
+    ##             eq/ax_con_int_list list of list/tuple of int e.g,  [[1,2] [1,2]]
     result_ax_full = list()
     result_eq_full = list()
     result_ax_con = list()
@@ -384,19 +578,22 @@ def generate_all_ligand_deltametrics(mol, loud, depth=4, name=False):
     allowed_strings = ['electronegativity', 'nuclear_charge', 'ident', 'topology', 'size']
     labels_strings = ['chi', 'Z', 'I', 'T', 'S']
     for ii, properties in enumerate(allowed_strings):
-        ax_ligand_ac_con, eq_ligand_ac_con = find_ligand_deltametrics_oct(mol, properties, loud, depth, name)
+        ax_ligand_ac_con, eq_ligand_ac_con = find_ligand_deltametrics_oct(mol, properties, loud, depth, name,oct=True,custom_ligand_dict=custom_ligand_dict)
         this_colnames = []
         for i in range(0, depth + 1):
             this_colnames.append(labels_strings[ii] + '-' + str(i))
         colnames.append(this_colnames)
         result_ax_con.append(ax_ligand_ac_con)
         result_eq_con.append(eq_ligand_ac_con)
-    # results_dictionary = {'colnames': colnames, 'result_ax_con_del': result_ax_con, 'result_eq_con_del': result_eq_con}
-    results_dictionary = {'colnames': colnames, 'result_ax_con': result_ax_con, 'result_eq_con': result_eq_con}
+    if flag_name:
+        results_dictionary = {'colnames': colnames, 'result_ax_con_del': result_ax_con,
+                              'result_eq_con_del': result_eq_con}
+    else:
+        results_dictionary = {'colnames': colnames, 'result_ax_con': result_ax_con, 'result_eq_con': result_eq_con}
     return results_dictionary
 
 
-def generate_metal_autocorrelations(mol, loud, depth=4, oct=True):
+def generate_metal_autocorrelations(mol, loud, depth=4, oct=True, flag_name=False):
     #	oct - bool, if complex is octahedral, will use better bond checks
     result = list()
     colnames = []
@@ -409,12 +606,47 @@ def generate_metal_autocorrelations(mol, loud, depth=4, oct=True):
             this_colnames.append(labels_strings[ii] + '-' + str(i))
         colnames.append(this_colnames)
         result.append(metal_ac)
-    # results_dictionary = {'colnames': colnames, 'results_mc_ac': result}
-    results_dictionary = {'colnames': colnames, 'results': result}
+    if flag_name:
+        results_dictionary = {'colnames': colnames, 'results_mc_ac': result}
+        # print(results_dictionary)
+    else:
+        results_dictionary = {'colnames': colnames, 'results': result}
     return results_dictionary
 
 
-def generate_metal_deltametrics(mol, loud, depth=4, oct=True):
+def generate_metal_ox_autocorrelations(oxmodifier, mol, loud, depth=4, oct=True, flag_name=False):
+    ## oxmodifier - dict, used to modify prop vector (e.g. for adding 
+    ##             ONLY used with  ox_nuclear_charge    ox or charge)
+    ##              {"Fe":2, "Co": 3} etc, normally only 1 metal... 
+    #	oct - bool, if complex is octahedral, will use better bond checks
+    result = list()
+    colnames = []
+    metal_ox_ac = metal_only_autocorrelation(mol, 'ox_nuclear_charge', depth, oct=oct,modifier=oxmodifier)
+    this_colnames = []
+    for i in range(0, depth + 1):
+            this_colnames.append('O' + '-' + str(i))        
+    colnames.append(this_colnames)
+    result.append(metal_ox_ac)
+    results_dictionary = {'colnames': colnames, 'results': result}
+    return results_dictionary
+
+def generate_metal_ox_deltametrics(oxmodifier, mol, loud, depth=4, oct=True, flag_name=False):
+    ## oxmodifier - dict, used to modify prop vector (e.g. for adding 
+    ##             ONLY used with  ox_nuclear_charge    ox or charge)
+    ##              {"Fe":2, "Co": 3} etc, normally only 1 metal... 
+    #	oct - bool, if complex is octahedral, will use better bond checks
+    result = list()
+    colnames = []
+    metal_ox_ac = metal_only_deltametric(mol, 'ox_nuclear_charge', depth, oct=oct,modifier=oxmodifier)
+    this_colnames = []
+    for i in range(0, depth + 1):
+        this_colnames.append('O' + '-' + str(i))
+    colnames.append(this_colnames)
+    result.append(metal_ox_ac)
+    results_dictionary = {'colnames': colnames, 'results': result}
+    return results_dictionary
+
+def generate_metal_deltametrics(mol, loud, depth=4, oct=True, flag_name=False):
     #	oct - bool, if complex is octahedral, will use better bond checks
     result = list()
     colnames = []
@@ -427,12 +659,14 @@ def generate_metal_deltametrics(mol, loud, depth=4, oct=True):
             this_colnames.append(labels_strings[ii] + '-' + str(i))
         colnames.append(this_colnames)
         result.append(metal_ac)
-    # results_dictionary = {'colnames': colnames, 'results_mc_del': result}
-    results_dictionary = {'colnames': colnames, 'results': result}
+    if flag_name:
+        results_dictionary = {'colnames': colnames, 'results_mc_del': result}
+    else:
+        results_dictionary = {'colnames': colnames, 'results': result}
     return results_dictionary
 
 
-def generate_full_complex_autocorrelations(mol, loud, depth=4, oct=True):
+def generate_full_complex_autocorrelations(mol, loud, depth=4, oct=True, flag_name=False):
     result = list()
     colnames = []
     allowed_strings = ['electronegativity', 'nuclear_charge', 'ident', 'topology', 'size']
@@ -444,8 +678,10 @@ def generate_full_complex_autocorrelations(mol, loud, depth=4, oct=True):
             this_colnames.append(labels_strings[ii] + '-' + str(i))
         colnames.append(this_colnames)
         result.append(metal_ac)
-    # results_dictionary = {'colnames': colnames, 'results_f_all': result}
-    results_dictionary = {'colnames': colnames, 'results': result}
+    if flag_name:
+        results_dictionary = {'colnames': colnames, 'results_f_all': result}
+    else:
+        results_dictionary = {'colnames': colnames, 'results': result}
     return results_dictionary
 
 
@@ -460,7 +696,7 @@ def generate_atomonly_autocorrelations(mol, atomIdx, loud, depth=4, oct=True):
     colnames = []
     allowed_strings = ['electronegativity', 'nuclear_charge', 'ident', 'topology', 'size']
     labels_strings = ['chi', 'Z', 'I', 'T', 'S']
-    print('The selected connection type is ' + str(mol.getAtom(atomIdx).symbol()))
+    #print('The selected connection type is ' + str(mol.getAtom(atomIdx).symbol()))
     for ii, properties in enumerate(allowed_strings):
         atom_only_ac = atom_only_autocorrelation(mol, properties, depth, atomIdx, oct=oct)
         this_colnames = []
