@@ -426,6 +426,7 @@ def substr_load(usersubstrate,subcores=None):
 #  @param licores Ligands dictionary (reloads if not specified - default, useful when using an externally modified dictionary)
 #  @return mol3D of ligand, error messages
 def lig_load(userligand,licores=None):
+    
     if licores == None:
         licores = getlicores()
         #@licores.pop("x", None)
@@ -445,9 +446,10 @@ def lig_load(userligand,licores=None):
         userligand = userligand.replace('~',homedir)
     emsg = False
     lig = mol3D() # initialize ligand molecule
+    lig.needsconformer = False
     ### check if ligand exists in dictionary
     if userligand in licores.keys():
-        print('loading ligand from dictionary')
+        print('loading ligand from dictionary: ' +str(userligand))
         dbentry = licores[userligand]
         # load lig mol file (with hydrogens)
         if globs.custom_path:
@@ -455,6 +457,7 @@ def lig_load(userligand,licores=None):
         else:
             flig = resource_filename(Requirement.parse("molSimplify"),"molSimplify/Ligands/" +dbentry[0])
         # check if ligand xyz/mol file exists
+        #print('looking for '+flig)
         if not glob.glob(flig):
             emsg = "We can't find the ligand structure file %s right now! Something is amiss. Exiting..\n" % flig
             print emsg
@@ -464,7 +467,10 @@ def lig_load(userligand,licores=None):
         elif ('.mol' in flig):
             lig.OBMol = lig.getOBMol(flig,'molf')
         elif ('.smi' in flig):
+            print('SMILES conversion')
             lig.OBMol = lig.getOBMol(flig,'smif')
+            lig.needsconformer = True
+
         ### modified the check for length,
         ### as it parsing string length instead of
         ### list length!
@@ -488,6 +494,7 @@ def lig_load(userligand,licores=None):
             lig.grps = []
         if len(dbentry) > 3:
             lig.ffopt = dbentry[4][0]
+            
     ### load from file
     elif ('.mol' in userligand or '.xyz' in userligand or '.smi' in userligand or '.sdf' in userligand):
         #flig = resource_filename(Requirement.parse("molSimplify"),"molSimplify/" +userligand)
@@ -525,6 +532,7 @@ def lig_load(userligand,licores=None):
             print emsg
             return False,emsg
         lig.ident = 'smi'
+        lig.needsconformer = True
     lig.name = userligand
     return lig,emsg
 
