@@ -1,9 +1,12 @@
 from molSimplify.Classes.mol3D import *
-import glob, os, time
+import glob, os, time, re, sys
 import numpy as np
 from molSimplify.Classes.globalvars import globalvars
 from molSimplify.Classes.ligand import *
 from molSimplify.Scripts.geometry import vecangle, distance
+import openbabel
+from molSimplify.Informatics.graph_analyze import obtain_truncation
+from molSimplify.Classes.atom3D import *
 
 # from openpyxl import load_workbook
 # from openpyxl import Workbook
@@ -12,22 +15,40 @@ from molSimplify.Scripts.geometry import vecangle, distance
 ### Check whether the complex form an octahedral.
 ### Written by Chenru Duan
 ### upload: 3/7/2018
-### update: 3/16/2018
+### update: 4/5/2018
 
-dict_oct_check_loose = {'rmsd_max': 0.4, 'atom_dist_max': 0.7,
+
+### Need more tests for the cutoffs just added for the linear ligands oreintation:
+### 1) 'devi_linear_avrg', 2) 'devi_linear_max'
+dict_oct_check_loose = {'rmsd_max': 0.4, 'atom_dist_max': 0.6,
                         'num_coord_metal': 6, 'oct_angle_devi_max': 15,
-                        'dist_del_eq': 0.45, 'dist_del_ax': 0.6,
-                        'dist_del_eq_ax': 0.8, 'max_del_sig_angle': 27}
+                        'dist_del_eq': 0.45, 'max_del_sig_angle': 27,
+                        'dist_del_all': 1.2,
+                        'devi_linear_avrg': 15, 'devi_linear_max': 28}
 
-dict_oct_check_st = {'rmsd_max': 0.3, 'atom_dist_max': 0.5,
+dict_oct_check_st = {'rmsd_max': 0.3, 'atom_dist_max': 0.45,
                      'num_coord_metal': 6, 'oct_angle_devi_max': 12,
                      'dist_del_eq': 0.35, 'dist_del_all': 1,
                      'max_del_sig_angle': 22.5,
-                     'dist_del_ax': 0.5, 'dist_del_eq_ax': 0.6}  # default cutoff
+                     'devi_linear_avrg': 10, 'devi_linear_max': 20}  # default cutoff
+
+dict_oneempty_check_st = {'rmsd_max': 0.4, 'atom_dist_max': 0.7,
+                          'num_coord_metal': 5, 'oct_angle_devi_max': 15,
+                          'dist_del_eq': 0.5, 'max_del_sig_angle': 18,
+                          'dist_del_all': 1,
+                          'devi_linear_avrg': 10, 'devi_linear_max': 20}
+
+dict_oneempty_check_loose = {'rmsd_max': 0.6, 'atom_dist_max': 0.9,
+                             'num_coord_metal': 5, 'oct_angle_devi_max': 20,
+                             'dist_del_eq': 0.6, 'max_del_sig_angle': 27,
+                             'dist_del_all': 1.2,
+                             'devi_linear_avrg': 15, 'devi_linear_max': 28}
 
 dict_staus = {'good': 1, 'bad': 0}
 
 oct_angle_ref = [[90, 90, 90, 90, 180] for x in range(6)]
+oneempty_angle_ref = [[90, 90, 90, 90], [180, 90, 90, 90], [180, 90, 90, 90],
+                      [180, 90, 90, 90], [180, 90, 90, 90]]
 
 
 ## input: a xyz file
@@ -38,27 +59,32 @@ def create_mol_with_xyz(_file_in):
     return my_mol
 
 
-### Deprecated
-# def comp_angle_arr(input_arr, target_arr):
-#     output_arr = []
-#     del_arr = []
-#     print('!!!!!!!!', input_arr)
-#     for idx, ele in enumerate(input_arr):
-#         del_abs = []
-#         for _ele in target_arr:
-#             del_abs.append(abs(ele - _ele))
-#         min_del = min(del_abs)
-#         del_arr.append([min_del, idx])
-#     del_arr.sort()
-#     sum_del = 0
-#     for idx in range(len(target_arr)):
-#         posi = del_arr[idx][1]
-#         sum_del += del_arr[idx][0]
-#         output_arr.append(input_arr[posi])
-#     output_arr.sort()
-#     sum_del = sum_del / len(target_arr)
-#     print('++++++', output_arr)
-#     return output_arr, sum_del
+def readfromtxt(mol, txt):
+    # print('!!!!', filename)
+    globs = globalvars()
+    en_dict = globs.endict()
+    mol.graph = []
+    for line in txt:
+        line_split = line.split()
+        if len(line_split) == 4 and line_split[0]:
+            # this looks for unique atom IDs in files
+            lm = re.search(r'\d+$', line_split[0])
+            # if the string ends in digits m will be a Match object, or None otherwise.
+            if lm is not None:
+                symb = re.sub('\d+', '', line_split[0])
+                # number = lm.group()
+                # # print('sym and number ' +str(symb) + ' ' + str(number))
+                # globs = globalvars()
+                atom = atom3D(symb, [float(line_split[1]), float(line_split[2]), float(line_split[3])],
+                              name=line_split[0])
+            elif line_split[0] in en_dict.keys():
+                atom = atom3D(line_split[0], [float(line_split[1]), float(line_split[2]), float(line_split[3])])
+            else:
+                print('cannot find atom type')
+                sys.exit()
+            mol.addAtom(atom)
+    return mol
+
 
 ## description: select the closest elements in input_arr compared
 ##              to the target array. Is used to screen atoms that
@@ -163,14 +189,14 @@ def sort_sec_ele(ele):
 ## construct an Oct.
 ## input: a xyz file
 ## output: coordination number for metal, and their indexes.
-def get_num_coord_metal(file_in):
+def get_num_coord_metal(file_in, debug=False):
     my_mol = create_mol_with_xyz(_file_in=file_in)
     metal_ind = my_mol.findMetal()[0]
     metal_coord = my_mol.getAtomCoords(metal_ind)
-    # print('metal index:', metal_ind)
-    print('metal coordinate:', metal_coord)
     catoms = my_mol.getBondedAtomsOct(ind=metal_ind)
-    print('coordinations: ', catoms, len(catoms))
+    if debug:
+        print('metal coordinate:', metal_coord)
+        print('coordinations: ', catoms, len(catoms))
     ## standard 1: number of coordination for metal.
     num_coord_metal = len(catoms)
     return num_coord_metal, catoms
@@ -182,47 +208,75 @@ def get_num_coord_metal(file_in):
 ## input: optimized and original xyz file.
 ## output: two scalar of maximum rmsd for ligands, and the
 ##         maximum distance change in ligands.
-def ligand_comp_org(file_in, file_init_geo, flag_deleteH=True, flag_loose=False,
-                    flag_lbd=True):
+def ligand_comp_org(file_in, file_init_geo, catoms_arr, flag_deleteH=True, flag_loose=False,
+                    flag_lbd=True, debug=False, depth=3, BondedOct=False):
     liglist, liglist_init, flag_match = match_lig_list(file_in, file_init_geo,
-                                                       flag_loose, flag_lbd)
-    print('lig_list:', liglist)
-    print('lig_list_init:', liglist_init)
+                                                       catoms_arr,
+                                                       flag_loose, flag_lbd,
+                                                       debug=debug, depth=depth,
+                                                       BondedOct=BondedOct)
+    if debug:
+        print('lig_list:', liglist, len(liglist))
+        print('lig_list_init:', liglist_init, len(liglist_init))
+    if flag_lbd:
+        mymol_xyz = 'mymol_trunc_tmp.xyz'
+        initmol_xyz = 'init_trunc_tmp.xyz'
+    else:
+        mymol_xyz = file_in
+        initmol_xyz = file_init_geo
     if flag_match:
         rmsd_arr, max_atom_dist_arr = [], []
         for idx, lig in enumerate(liglist):
-            print('----This is %d th piece of ligand.' % (idx + 1))
-            posi_shift = 2
-            start_posi = posi_shift + lig[0]
-            end_posi = posi_shift + lig[len(lig) - 1]
             lig_init = liglist_init[idx]
-            start_posi_init = posi_shift + lig_init[0]
-            end_posi_init = posi_shift + lig_init[len(lig_init) - 1]
-            with open(file_in, 'r') as fo:
-                lines = fo.readlines()[start_posi:end_posi + 1]
-                with open('tmp.xyz', 'w') as foo:
-                    foo.write('number of atoms not necessary\n')
-                    foo.write('tmp file for ligands comparison\n')
-                    foo.write(''.join(lines))
-            with open(file_init_geo, 'r') as fo:
-                lines = fo.readlines()[start_posi_init:end_posi_init + 1]
-                with open('tmp_org.xyz', 'w') as foo:
-                    foo.write('number of atoms not necessary\n')
-                    foo.write('tmp file for ligands comparison\n')
-                    foo.write(''.join(lines))
-            tmp_mol = create_mol_with_xyz('tmp.xyz')
-            tmp_org_mol = create_mol_with_xyz('tmp_org.xyz')
-            print('# atoms: %d, init: %d' % (tmp_mol.natoms, tmp_org_mol.natoms))
+            if debug:
+                print('----This is %d th piece of ligand.' % (idx + 1))
+                print('ligand is:', lig, lig_init)
+            posi_shift = 2
+            ## Create mol3D without a tmp file.
+            _start = time.clock()
+            with open(mymol_xyz, 'r') as fo:
+                foo = []
+                for ii, line in enumerate(fo):
+                    if (ii - posi_shift) in lig:
+                        if debug:
+                            print('line is', line)
+                        foo.append(line)
+            tmp_mol = mol3D()
+            tmp_mol = readfromtxt(tmp_mol, foo)
+            with open(initmol_xyz, 'r') as fo:
+                foo = []
+                for ii, line in enumerate(fo):
+                    if (ii - posi_shift) in lig:
+                        if debug:
+                            print('line is', line)
+                        foo.append(line)
+            tmp_org_mol = mol3D()
+            tmp_org_mol = readfromtxt(tmp_org_mol, foo)
+            _elapsed = (time.clock() - _start)
+            # print('-reading txt:', _elapsed)
+            if debug:
+                print('# atoms: %d, init: %d' % (tmp_mol.natoms, tmp_org_mol.natoms))
+                print('!!!!atoms:', [x.symbol() for x in tmp_mol.getAtoms()],
+                      [x.symbol() for x in tmp_org_mol.getAtoms()])
             if flag_deleteH:
                 tmp_mol.deleteHs()
                 tmp_org_mol.deleteHs()
+            # ---openbabel alignment
+            # tmp_org_mol.convert2OBMol()
+            # tmp_mol.convert2OBMol()
+            # OBAlign = openbabel.OBAlign(tmp_org_mol.OBMol,tmp_mol.OBMol)
+            # alignstatus = OBAlign.Align()
+            # print('did align succeed ' + str(alignstatus))
+            # rmsd = OBAlign.GetRMSD()
+            # print('RMSD is ' + str(rmsd))
             mol0, U, d0, d1 = kabsch(tmp_org_mol, tmp_mol)
             rmsd = tmp_mol.rmsd(tmp_org_mol)
             rmsd_arr.append(rmsd)
-            print('rmsd:', rmsd)
             atom_dist_max = tmp_mol.maxatomdist(tmp_org_mol)
             max_atom_dist_arr.append(atom_dist_max)
-            print('atom_dist_max', atom_dist_max)
+            if debug:
+                print('rmsd:', rmsd)
+                print('atom_dist_max', atom_dist_max)
         rmsd_max = max(rmsd_arr)
         atom_dist_max = max(max_atom_dist_arr)
     else:
@@ -234,36 +288,47 @@ def ligand_comp_org(file_in, file_init_geo, flag_deleteH=True, flag_loose=False,
 ## useful for cases where the init geo and opt geo have the
 ## different ligands arrangement (when you only have the opt geo
 ## but still want init geo)
-def match_lig_list(file_in, file_init_geo, flag_loose, flag_lbd=True):
+def match_lig_list(file_in, file_init_geo, catoms_arr,
+                   flag_loose, flag_lbd=True, debug=False,
+                   depth=3, BondedOct=False):
     flag_match = True
     my_mol = create_mol_with_xyz(_file_in=file_in)
-    # print('natoms: ', my_mol.natoms)
-    print('In match, flag_loose', flag_loose)
     init_mol = create_mol_with_xyz(_file_in=file_init_geo)
-    liglist_init, ligdents_init, ligcons_init = ligand_breakdown(init_mol)
-    if flag_lbd: ## Also do ligand breakdown for opt geo
-        liglist, ligdents, ligcons = ligand_breakdown(my_mol, flag_loose)
-    else: ## ceate/use the liglist, ligdents, ligcons of initial geo as we just wanna track them down
+    if flag_lbd:  ## Also do ligand breakdown for opt geo
+        my_mol_trunc = obtain_truncation(my_mol, catoms_arr, depth)
+        init_mol_trunc = obtain_truncation(init_mol, catoms_arr, depth)
+        my_mol_trunc.createMolecularGraph()
+        init_mol_trunc.createMolecularGraph()
+        init_mol_trunc.writexyz('init_trunc_tmp.xyz')
+        my_mol_trunc.writexyz('mymol_trunc_tmp.xyz')
+        liglist_init, ligdents_init, ligcons_init = ligand_breakdown(init_mol_trunc)
+        liglist, ligdents, ligcons = ligand_breakdown(my_mol_trunc, flag_loose)
+        liglist_atom = [[my_mol_trunc.getAtom(x).symbol() for x in ele]
+                        for ele in liglist]
+        liglist_init_atom = [[init_mol_trunc.getAtom(x).symbol() for x in ele]
+                             for ele in liglist_init]
+        if debug:
+            print('!!!!:', [x.symbol() for x in init_mol_trunc.getAtoms()])
+            print('liglist_init, ligdents_init, ligcons_init', liglist_init, ligdents_init, ligcons_init)
+    else:  ## ceate/use the liglist, ligdents, ligcons of initial geo as we just wanna track them down
+        # _start = time.clock()
+        if debug:
+            print('Just inherit the ligand list from init structure.')
+        liglist_init, ligdents_init, ligcons_init = ligand_breakdown(init_mol,
+                                                                     flag_loose=flag_loose,
+                                                                     BondedOct=BondedOct)
+        # _elapsed = (time.clock() - _start)
+        # print('time on lig_breakdoen:', _elapsed)
         liglist, ligdents, ligcons = liglist_init[:], ligdents_init[:], ligcons_init[:]
-    liglist_atom = [[my_mol.getAtom(x).symbol() for x in ele]
-                    for ele in liglist]
-    liglist_init_atom = [[init_mol.getAtom(x).symbol() for x in ele]
-                         for ele in liglist_init]
-    print('ligand_list opt in symbols:', liglist_atom)
-    print('ligand_list init in symbols: ', liglist_init_atom)
-    liglist_shifted = []
-    ## --- match opt geo with init geo---
-    # for ele in liglist_atom:
-    #     try:
-    #         posi = liglist_init_atom.index(ele)
-    #         liglist_init_shifted.append(liglist_init[posi])
-    #         liglist_init_atom.pop(posi)
-    #         liglist_init.pop(posi)
-    #     except ValueError:
-    #         print('Ligands cannot match!')
-    #         flag_match = False
+        liglist_atom = [[my_mol.getAtom(x).symbol() for x in ele]
+                        for ele in liglist]
+        liglist_init_atom = [[init_mol.getAtom(x).symbol() for x in ele]
+                             for ele in liglist_init]
 
-    ## --- match init geo with opt geo---
+    if debug:
+        print('ligand_list opt in symbols:', liglist_atom)
+        print('ligand_list init in symbols: ', liglist_init_atom)
+    liglist_shifted = []
     for ele in liglist_init_atom:
         try:
             posi = liglist_atom.index(ele)
@@ -271,11 +336,78 @@ def match_lig_list(file_in, file_init_geo, flag_loose, flag_lbd=True):
             liglist_atom.pop(posi)
             liglist.pop(posi)
         except ValueError:
-            print('Ligands cannot match!')
+            if debug:
+                print('Ligands cannot match!')
             flag_match = False
+    if debug:
+        print('!!!!!returns', liglist_shifted, liglist_init)
     return liglist_shifted, liglist_init, flag_match
 
 
+def find_the_other_ind(arr, ind):
+    arr.pop(arr.index(ind))
+    return arr[0]
+
+## The linear ligand here ius defined as:
+## 1) ligands that contains only two atoms. For example CO
+## 2) ligands with more than two atoms and looks linear itself.
+## For example: A-B-C-.... with the angle of A-B and B-C larger than 170.
+def is_linear_ligand(mol, ind):
+    catoms = mol.getBondedAtomsSmart(ind)
+    metal_ind = mol.findMetal()[0]
+    flag = False
+    if metal_ind in catoms and len(catoms) == 2:
+        ind_next = find_the_other_ind(catoms[:], metal_ind)
+        _catoms = mol.getBondedAtomsSmart(ind_next)
+        if len(_catoms) == 1:
+            flag = True
+        elif len(_catoms) == 2:
+            ind_next2 = find_the_other_ind(_catoms[:], ind)
+            vec1 = np.array(mol.getAtomCoords(ind)) - np.array(mol.getAtomCoords(ind_next))
+            vec2 = np.array(mol.getAtomCoords(ind_next2)) - np.array(mol.getAtomCoords(ind_next))
+            ang = vecangle(vec1, vec2)
+            if ang > 170:
+                flag = True
+    # print(flag, catoms)
+    return flag, catoms
+
+
+def get_linear_angle(mol, ind):
+    flag, catoms = is_linear_ligand(mol, ind)
+    if flag:
+        vec1 = np.array(mol.getAtomCoords(catoms[0])) - np.array(mol.getAtomCoords(ind))
+        vec2 = np.array(mol.getAtomCoords(catoms[1])) - np.array(mol.getAtomCoords(ind))
+        ang = vecangle(vec1, vec2)
+    else:
+        ang = 0
+    return flag, ang
+
+
+def check_angle_linear(file_in, catoms_arr):
+    mol = create_mol_with_xyz(file_in)
+    dict_angle_linear = {}
+    for ind in catoms_arr:
+        flag, ang = get_linear_angle(mol, ind)
+        dict_angle_linear[str(ind)] = [flag, ang]
+    dict_orientation = {}
+    devi_linear_avrg, devi_linear_max = 0, 0
+    count = 0
+    for key in dict_angle_linear:
+        [flag, ang] = dict_angle_linear[key]
+        if flag:
+            count += 1
+            devi_linear_avrg += 180 - ang
+            if (180 - ang) > devi_linear_max:
+                devi_linear_max = 180 - ang
+    if count:
+        devi_linear_avrg /= count
+    else:
+        devi_linear_avrg = 0
+    dict_orientation['devi_linear_avrg'] = devi_linear_avrg
+    dict_orientation['devi_linear_max'] = devi_linear_max
+    return dict_angle_linear, dict_orientation
+
+
 ## standard 3: catoms structure compared to a perfect octahedral.
 ## Note: The same logic can be used to judge whether a complex is
 ##       Tetrahedron or panor square.
@@ -285,7 +417,8 @@ def match_lig_list(file_in, file_init_geo, flag_loose, flag_lbd=True):
 ##         for the metal. A metric for the distance deviation from the
 ##         pefect Oct, including the diff in eq ligands, ax ligands and
 ##         eq-ax ligands.
-def oct_comp(file_in, angle_ref=oct_angle_ref, catoms_arr=None):
+def oct_comp(file_in, angle_ref=oct_angle_ref, catoms_arr=None,
+             debug=False):
     my_mol = create_mol_with_xyz(_file_in=file_in)
     num_coord_metal, catoms = get_num_coord_metal(file_in=file_in)
     # metal_ind = my_mol.findMetal()[0]
@@ -313,10 +446,11 @@ def oct_comp(file_in, angle_ref=oct_angle_ref, catoms_arr=None):
         # print('The adjusted array is: ', out_theta)
         # theta_arr.append([catoms[idx1], sum_del, out_theta])
     th_output_arr, sum_del_angle, catoms_arr, max_del_sig_angle = loop_target_angle_arr(th_input_arr, angle_ref)
-    print('th:', th_output_arr)
-    print('sum_del:', sum_del_angle)
-    print('catoms_arr:', catoms_arr)
-    print('catoms_type:', [my_mol.getAtom(x).symbol() for x in catoms_arr])
+    if debug:
+        print('th:', th_output_arr)
+        print('sum_del:', sum_del_angle)
+        print('catoms_arr:', catoms_arr)
+        print('catoms_type:', [my_mol.getAtom(x).symbol() for x in catoms_arr])
     for idx, ele in enumerate(th_output_arr):
         theta_arr.append([catoms_arr[idx], sum_del_angle[idx], ele])
     # theta_arr.sort(key=sort_sec_ele)
@@ -327,8 +461,9 @@ def oct_comp(file_in, angle_ref=oct_angle_ref, catoms_arr=None):
     oct_catoms = theta_trunc_arr_T[0]
     oct_angle_devi = theta_trunc_arr_T[1]
     oct_angle_all = theta_trunc_arr_T[2]
-    print('Summation of deviation angle for catoms:', oct_angle_devi)
-    print('Angle for catoms:', oct_angle_all)
+    if debug:
+        print('Summation of deviation angle for catoms:', oct_angle_devi)
+        print('Angle for catoms:', oct_angle_all)
     for atom in oct_catoms:
         coord = catom_coord[catoms.index(atom)]
         dist = distance(coord, metal_coord)
@@ -350,17 +485,48 @@ def oct_comp(file_in, angle_ref=oct_angle_ref, catoms_arr=None):
         else:
             dist_ax, dist_eq = oct_dist[4:], oct_dist[:4]  # eq dist is smaller
     dist_del_all = oct_dist[-1] - oct_dist[0]
-    print('dist:', dist_eq, dist_ax)
+    if debug:
+        print('dist:', dist_eq, dist_ax)
     dist_del_eq = max(dist_eq) - min(dist_eq)
     dist_del_ax = max(dist_ax) - min(dist_ax)
     dist_del_eq_ax = max(abs(max(dist_eq) - min(dist_ax)), abs(max(dist_ax) - min(dist_eq)))
     oct_dist_del = [dist_del_eq, dist_del_ax, dist_del_eq_ax, dist_del_all]
-    print('distance difference for catoms to metal (eq, ax, eq_ax):', oct_dist_del)
+    if debug:
+        print('distance difference for catoms to metal (eq, ax, eq_ax):', oct_dist_del)
     return oct_angle_devi, oct_dist_del, max_del_sig_angle, catoms_arr
 
 
+def dict_check_processing(dict_info, dict_check, std_not_use,
+                          num_coord=6, debug=False):
+    if debug:
+        print('dict_oct_info', dict_info)
+    for ele in std_not_use:
+        dict_info[ele] = 'banned_by_user'
+    flag_list = []
+    for key, values in dict_check.items():
+        if not dict_info[key] == 'banned_by_user':
+            if dict_info[key] > values:
+                flag_list.append(key)
+    if dict_info['num_coord_metal'] < num_coord:
+        flag_list.append('num_coord_metal')
+    if flag_list == ['num_coord_metal'] and \
+            (dict_info['num_coord_metal'] == -1 or dict_info['num_coord_metal'] > num_coord):
+        dict_info['num_coord_metal'] = num_coord
+        flag_list.remove('num_coord_metal')
+    if not len(flag_list):
+        flag_oct = 1  # good structure
+        flag_list = 'None'
+    else:
+        flag_oct = 0
+        flag_list = ', '.join(flag_list)
+        print('------bad structure!-----')
+        print('flag_list:', flag_list)
+    return flag_oct, flag_list, dict_info
+
+
 def Oct_inspection(file_in, file_init_geo=None, catoms_arr=None, dict_check=dict_oct_check_st,
-                   std_not_use=[], angle_ref=oct_angle_ref, flag_loose=True, flag_lbd=False):
+                   std_not_use=[], angle_ref=oct_angle_ref, flag_loose=True, flag_lbd=False,
+                   dict_check_loose=dict_oct_check_loose, BondedOct=True, debug=False):
     if catoms_arr == None:
         print('Error, must have ctoms! If not, please use IsOct.')
         quit()
@@ -370,14 +536,36 @@ def Oct_inspection(file_in, file_init_geo=None, catoms_arr=None, dict_check=dict
     num_coord_metal = 6
     oct_angle_devi, oct_dist_del, max_del_sig_angle = [-1, -1], [-1, -1, -1, -1], -1
     rmsd_max, atom_dist_max = -1, -1
+    dict_orientation = {'devi_linear_max': -1, 'devi_linear_avrg': -1}
     if not file_init_geo == None:
-        print('!!!Inspection,flag_loose:', flag_loose)
-        rmsd_max, atom_dist_max = ligand_comp_org(file_in, file_init_geo, flag_loose=flag_loose,flag_lbd=flag_lbd)
+        # print('!!!Inspection,flag_loose:', flag_loose)
+        # _start = time.clock()
+        rmsd_max, atom_dist_max = ligand_comp_org(file_in, file_init_geo,
+                                                  flag_loose=flag_loose,
+                                                  flag_lbd=flag_lbd,
+                                                  catoms_arr=catoms_arr,
+                                                  debug=debug,
+                                                  BondedOct=BondedOct)
+        # _elapsed = time.clock() - _start
+        # print('Time on ligand_comp_org:', _elapsed)
     if not rmsd_max == 'lig_mismatch':
-        oct_angle_devi, oct_dist_del, max_del_sig_angle, catoms_arr = oct_comp(file_in, angle_ref, catoms_arr)
+        # _start = time.clock()
+        oct_angle_devi, oct_dist_del, max_del_sig_angle, catoms_arr = oct_comp(file_in, angle_ref, catoms_arr,
+                                                                               debug=debug)
+        # _elapsed = time.clock() - _start
+        # print('Time on oct_comp:', _elapsed)
     else:
         num_coord_metal = -1
         rmsd_max, atom_dist_max = -1, -1
+        print('!!!!!Should always match. WRONG!!!!!')
+        quit()
+    dict_angle_linear, dict_orientation = check_angle_linear(file_in, catoms_arr)
+    if debug:
+        print('-------This is for the linear ligand orientation test-----')
+        print('!!!catoms_arr:', catoms_arr)
+        print('!!!dict_angle_linear', dict_angle_linear)
+        print('!!!dict_orientation', dict_orientation)
+        print('---------orientation end.------------')
     dict_oct_info = {}
     dict_oct_info['num_coord_metal'] = num_coord_metal
     dict_oct_info['rmsd_max'] = rmsd_max
@@ -388,23 +576,16 @@ def Oct_inspection(file_in, file_init_geo=None, catoms_arr=None, dict_check=dict
     dict_oct_info['dist_del_ax'] = oct_dist_del[1]
     dict_oct_info['dist_del_eq_ax'] = oct_dist_del[2]
     dict_oct_info['dist_del_all'] = oct_dist_del[3]
-    print('dict_oct_info', dict_oct_info)
-    for ele in std_not_use:
-        dict_oct_info[ele] = 'banned_by_user'
-    flag_list = []
-    for key, values in dict_check.items():
-        if not dict_oct_info[key] == 'banned_by_user':
-            if dict_oct_info[key] > values:
-                flag_list.append(key)
-    if not len(flag_list):
-        flag_oct = 1  # good structure
-        flag_list = 'None'
-    else:
-        flag_oct = 0
-        flag_list = ', '.join(flag_list)
-        print('------bad structure!-----')
-        print('flag_list:', flag_list)
-    return flag_oct, flag_list, dict_oct_info
+    dict_oct_info.update(dict_orientation)
+    flag_oct, flag_list, dict_oct_info = dict_check_processing(dict_oct_info,
+                                                               dict_check=dict_check,
+                                                               std_not_use=std_not_use,
+                                                               num_coord=6, debug=debug)
+    flag_oct_loose, flag_list_loose, __ = dict_check_processing(dict_oct_info,
+                                                                dict_check=dict_check_loose,
+                                                                std_not_use=std_not_use,
+                                                                num_coord=6, debug=debug)
+    return flag_oct, flag_list, dict_oct_info, flag_oct_loose, flag_list_loose
 
 
 ## See whether a complex is Oct or not.
@@ -413,28 +594,33 @@ def Oct_inspection(file_in, file_init_geo=None, catoms_arr=None, dict_check=dict
 ##         dict_oct_info: values for each metric we check.
 def IsOct(file_in, file_init_geo=None, dict_check=dict_oct_check_st,
           std_not_use=[], angle_ref=oct_angle_ref, flag_catoms=False,
-          catoms_arr=None):
-    num_coord_metal, catoms = get_num_coord_metal(file_in)
+          catoms_arr=None, debug=False):
+    num_coord_metal, catoms = get_num_coord_metal(file_in, debug=debug)
     if not catoms_arr == None:
         catoms = catoms_arr
         num_coord_metal = len(catoms_arr)
 
-    # if file_init_geo != None and num_coord_metal>=6:
-    #     rmsd_max, atom_dist_max = ligand_comp_org(file_in, file_init_geo)
-    # else:
-    #     rmsd_max, atom_dist_max = -1, -1
     oct_angle_devi, oct_dist_del, max_del_sig_angle = [-1, -1], [-1, -1, -1, -1], -1
     rmsd_max, atom_dist_max = -1, -1
     catoms_arr = catoms
+    dict_orientation = {'devi_linear_max': -1, 'devi_linear_avrg': -1}
     if num_coord_metal >= 6:
-        if not file_init_geo == None:
-            rmsd_max, atom_dist_max = ligand_comp_org(file_in, file_init_geo)
         if not rmsd_max == 'lig_mismatch':
             num_coord_metal = 6
-            oct_angle_devi, oct_dist_del, max_del_sig_angle, catoms_arr = oct_comp(file_in, angle_ref, catoms_arr)
+            oct_angle_devi, oct_dist_del, max_del_sig_angle, catoms_arr = oct_comp(file_in, angle_ref,
+                                                                                   catoms_arr, debug=debug)
+        if not file_init_geo == None:
+            rmsd_max, atom_dist_max = ligand_comp_org(file_in, file_init_geo, catoms_arr, debug=debug)
         else:
-            num_coord_metal = -1
+            # num_coord_metal = -1
             rmsd_max, atom_dist_max = -1, -1
+        dict_angle_linear, dict_orientation = check_angle_linear(file_in, catoms_arr)
+        if debug:
+            print('-------This is for the linear ligand orientation test-----')
+            print('!!!catoms_arr:', catoms_arr)
+            print('!!!dict_angle_linear', dict_angle_linear)
+            print('!!!dict_orientation', dict_orientation)
+            print('---------orientation end.------------')
     dict_oct_info = {}
     dict_oct_info['num_coord_metal'] = num_coord_metal
     dict_oct_info['rmsd_max'] = rmsd_max
@@ -442,106 +628,55 @@ def IsOct(file_in, file_init_geo=None, dict_check=dict_oct_check_st,
     dict_oct_info['oct_angle_devi_max'] = max(oct_angle_devi)
     dict_oct_info['max_del_sig_angle'] = max_del_sig_angle
     dict_oct_info['dist_del_eq'] = oct_dist_del[0]
-    dict_oct_info['dist_del_ax'] = oct_dist_del[1]
-    dict_oct_info['dist_del_eq_ax'] = oct_dist_del[2]
     dict_oct_info['dist_del_all'] = oct_dist_del[3]
-    print('dict_oct_info', dict_oct_info)
-    for ele in std_not_use:
-        dict_oct_info[ele] = 'banned_by_user'
-    flag_list = []
-    ## ---Adjust cutoff value---
-    # if dict_oct_info['rmsd_max'] > 0.3:
-    #     flag_list.append('rmsd_max')
-    # if dict_oct_info['atom_dist_max'] > 0.5:
-    #     flag_list.append('atom_dist_max')
-    # if dict_oct_info['num_coord_metal'] < 6:
-    #     flag_list.append('num_coord_metal')
-    # if dict_oct_info['oct_angle_devi_max'] > 10:
-    #     flag_list.append('oct_angle_devi_max')
-    # if dict_oct_info['dist_del_eq'] > 0.35:
-    #     flag_list.append('dist_del_eq')
-    # if dict_oct_info['dist_del_ax'] > 0.4:
-    #     flag_list.append('dist_del_ax')
-    # if dict_oct_info['dist_del_eq_ax'] > 0.8:
-    #     flag_list.append('dist_del_eq_ax')
-
-    for key, values in dict_check.items():
-        if not dict_oct_info[key] == 'banned_by_user':
-            if dict_oct_info[key] > values:
-                flag_list.append(key)
-    if num_coord_metal < 6:
-        flag_list.append('num_coord_metal')
-    # if num_coord_metal == -1:
-    #     flag_list.remove('rmsd_max')
-    #     flag_list.remove('atom_dist_max')
-    ## Case when the num_coord_metal > 6 but still forms a octahedral.
-    # if ('num_coord_metal' in flag_list) and (not 'oct_angle_devi_max' in flag_list) and \
-    #         (not 'dist_del_eq' in flag_list) and (not 'dist_del_ax' in flag_list) and \
-    #         (not 'dist_del_eq_ax' in flag_list):
-    #     num_coord_metal = 6
-    #     flag_list.remove('num_coord_metal')
-
-    if not len(flag_list):
-        flag_oct = 1  # good structure
-        flag_list = 'None'
-    else:
-        flag_oct = 0
-        flag_list = ', '.join(flag_list)
-        print('------bad structure!-----')
-        print('flag_list:', flag_list)
+    dict_oct_info.update(dict_orientation)
+    flag_oct, flag_list, dict_oct_info = dict_check_processing(dict_oct_info, dict_check,
+                                                               std_not_use, num_coord=6,
+                                                               debug=debug)
     if not flag_catoms:
         return flag_oct, flag_list, dict_oct_info
     else:
         return flag_oct, flag_list, dict_oct_info, catoms_arr
 
 
-def IsStructure(file_in, file_init_geo=None, dict_check=dict_oct_check_st,
-                std_not_use=[], angle_ref=oct_angle_ref, num_coord=6):
-    num_coord_metal, catoms = get_num_coord_metal(file_in)
+def IsStructure(file_in, file_init_geo=None, dict_check=dict_oneempty_check_st,
+                std_not_use=[], angle_ref=oneempty_angle_ref, num_coord=5,
+                flag_catoms=False, debug=False):
+    num_coord_metal, catoms = get_num_coord_metal(file_in, debug=debug)
 
-    if file_init_geo != None:
-        rmsd_max, atom_dist_max = ligand_comp_org(file_in, file_init_geo)
-    else:
-        rmsd_max, atom_dist_max = -1, -1
+    struct_angle_devi, struct_dist_del, max_del_sig_angle = [-1, -1], [-1, -1, -1, -1], -1
+    rmsd_max, atom_dist_max = -1, -1
+    dict_orientation = {'devi_linear_max': -1, 'devi_linear_avrg': -1}
     if num_coord_metal >= num_coord:
-        struct_angle_devi, struct_dist_del, max_del_sig_angle, catoms_arr = oct_comp(file_in, angle_ref)
-    else:
-        struct_angle_devi, struct_dist_del, max_del_sig_angle = [-1, -1], [-1, -1, -1, -1], -1
+        struct_angle_devi, struct_dist_del, max_del_sig_angle, catoms_arr = oct_comp(file_in, angle_ref,
+                                                                                     debug=debug)
+        if file_init_geo != None:
+            rmsd_max, atom_dist_max = ligand_comp_org(file_in, file_init_geo, catoms_arr, debug=debug)
+        else:
+            rmsd_max, atom_dist_max = -1, -1
+        dict_angle_linear, dict_orientation = check_angle_linear(file_in, catoms_arr)
+        if debug:
+            print('-------This is for the linear ligand orientation test-----')
+            print('!!!catoms_arr:', catoms_arr)
+            print('!!!dict_angle_linear', dict_angle_linear)
+            print('!!!dict_orientation', dict_orientation)
+            print('---------orientation end.------------')
     dict_struct_info = {}
     dict_struct_info['num_coord_metal'] = num_coord_metal
     dict_struct_info['rmsd_max'] = rmsd_max
     dict_struct_info['atom_dist_max'] = atom_dist_max
-    dict_struct_info['struct_angle_devi_max'] = max(struct_angle_devi)
+    dict_struct_info['oct_angle_devi_max'] = max(struct_angle_devi)
     dict_struct_info['max_del_sig_angle'] = max_del_sig_angle
     dict_struct_info['dist_del_eq'] = struct_dist_del[0]
-    dict_struct_info['dist_del_ax'] = struct_dist_del[1]
-    dict_struct_info['dist_del_eq_ax'] = struct_dist_del[2]
     dict_struct_info['dist_del_all'] = struct_dist_del[3]
-    print('dict_struct_info', dict_struct_info)
-    for ele in std_not_use:
-        dict_struct_info[ele] = 'banned_by_user'
-    flag_list = []
-
-    for key, values in dict_check.items():
-        if not dict_struct_info[key] == 'banned_by_user':
-            if dict_struct_info[key] > values:
-                flag_list.append(key)
-    ## Case when the num_coord_metal > 6 but still forms a structahedral.
-    if ('num_coord_metal' in flag_list) and (not 'struct_angle_devi_max' in flag_list) and \
-            (not 'dist_del_eq' in flag_list) and (not 'dist_del_ax' in flag_list) and \
-            (not 'dist_del_eq_ax' in flag_list):
-        num_coord_metal = num_coord
-        flag_list.remove('num_coord_metal')
-
-    if not len(flag_list):
-        flag_struct = 1  # good structure
-        flag_list = 'None'
+    dict_struct_info.update(dict_orientation)
+    flag_struct, flag_list, dict_struct_info = dict_check_processing(dict_struct_info, dict_check,
+                                                                     std_not_use, num_coord=num_coord,
+                                                                     debug=debug)
+    if not flag_catoms:
+        return flag_struct, flag_list, dict_struct_info
     else:
-        flag_struct = 0
-        flag_list = ', '.join(flag_list)
-        print('------bad structure!-----')
-        print('flag_list:', flag_list)
-    return flag_struct, flag_list, dict_struct_info
+        return flag_struct, flag_list, dict_struct_info, catoms_arr
 
 
 ## input: _path: path for opt geo
@@ -593,8 +728,6 @@ def gen_file_with_name(path_init_geo, name_opt):
     name_opt = name_opt.split('_')
     name_opt = '_'.join(name_opt[:len(name_opt) - 1])
     name_init = '%s/%s_mols.xyz' % (path_init_geo, name_opt)
-    return name_init
-
 ###------------DEMO-------------------
 ## ---batch processing----
 # _path = './fe_optimized_geometry'
