@@ -294,7 +294,6 @@ def init_ligand(args,lig,tcats,keepHs,i):
             lig.cat = [lig.natoms]
         else:
             lig.cat = tcats[i]
-
     # change name
     lig3D = mol3D()
     lig3D.copymol3D(lig)
@@ -313,6 +312,7 @@ def init_ligand(args,lig,tcats,keepHs,i):
     if args.ff and 'b' in args.ffoption and not rempi:
         if 'b' in lig.ffopt.lower():
             print 'FF optimizing ligand'
+            lig3D.convert2mol3D()
             lig3D,enl = ffopt(args.ff,lig3D,lig3D.cat,0,[],False,[],100,args.debug)
     # skip hydrogen removal for pi-coordinating ligands
     if not rempi:
@@ -343,8 +343,23 @@ def init_ligand(args,lig,tcats,keepHs,i):
                 lig3D.charge = lig3D.charge - 1
     # Conformer search for multidentate SMILES ligands
     lig3D.convert2OBMol()
+    
+    if lig.needsconformer:
+        tcats[i] = True
+        print('getting conformers for ' + str(lig.ident))
+
     if len(lig.cat) > 1 and tcats[i]:
-        lig3D = GetConf(lig3D,lig.cat)    # check if ligand should decorated
+        print('generating conformations')
+        # loop  over conformation gen until success or break 
+        breaker = False
+        count = 0
+        while (not breaker) and count <= 5:
+            try:
+                lig3D = GetConf(lig3D,lig.cat)    # check if ligand should decorated
+                breaker = True            
+            except:
+                count += 1
+                print('lig conformer input failed ' + str(count)  + ' times, trying again...')
     return lig3D,rempi,ligpiatoms
 
 ## Distorts backbone according to user specified angles
@@ -426,7 +441,7 @@ def ffopt(ff,mol,connected,constopt,frozenats,frozenangles,mlbonds,nsteps,debug=
     ffav = 'mmff94, uff, ghemical, gaff, mmff94s' # force fields
     if ff.lower() not in ffav:
         print 'Requested force field not available. Defaulting to MMFF94'
-        ff = 'mmff94'
+        ff = 'uff'
     # perform constrained ff optimization if requested after #
     if (constopt > 0):
         # get metal
@@ -456,24 +471,31 @@ def ffopt(ff,mol,connected,constopt,frozenats,frozenangles,mlbonds,nsteps,debug=
                 constr.AddAtomConstraint(catom+1) # indexing babel
             else:
                 constr.AddDistanceConstraint(midx+1,catom+1,mlbonds[ii]) # indexing babel
-        bridgingatoms = []
-        # identify bridging atoms in the case of bimetallic cores, as well as single-atom ligands (oxo, nitrido)
-        # these are immune to deletion
-        for i in range(mol.natoms):
-            nbondedmetals = len([idx for idx in range(len(mol.getBondedAtoms(i))) if mol.getAtom(mol.getBondedAtoms(i)[idx]).ismetal()])
-            if nbondedmetals > 1 or (nbondedmetals == 1 and len(mol.getBondedAtoms(i)) == 1):
-                bridgingatoms.append(i)
-        # ensure correct valences for FF setup
-        for m in indmtls:
+        #print('ff is '+ str(ff))        
+        if not ff.lower() == "uff":
+            bridgingatoms = []
+            # identify bridging atoms in the case of bimetallic cores, as well as single-atom ligands (oxo, nitrido)
+            # these are immune to deletion
+            for i in range(mol.natoms):
+                nbondedmetals = len([idx for idx in range(len(mol.getBondedAtoms(i))) if mol.getAtom(mol.getBondedAtoms(i)[idx]).ismetal()])
+                if nbondedmetals > 1 or (nbondedmetals == 1 and len(mol.getBondedAtoms(i)) == 1):
+                    bridgingatoms.append(i)
+            # ensure correct valences for FF setup
+            deleted_bonds = 0
+            
+            for m in indmtls:
             # first delete all metal-ligand bonds excluding bridging atoms
-            for i in range(len(mol.getBondedAtoms(m))):
-                if OBMol.GetBond(m+1,mol.getBondedAtoms(m)[i]+1) is not None and mol.getBondedAtoms(m)[i] not in bridgingatoms:
-                    OBMol.DeleteBond(OBMol.GetBond(m+1,mol.getBondedAtoms(m)[i]+1))
-            # then add back one metal-ligand bond for FF
-            if OBMol.GetAtom(m+1).GetValence() == 0:
-                for i in mol.getBondedAtomsOct(m,1+len(bridgingatoms)):
-                    if OBMol.GetAtom(m+1).GetValence() < 1 and i not in bridgingatoms:
-                        OBMol.AddBond(m+1,i+1,1)
+                for i in range(len(mol.getBondedAtoms(m))):
+                    if OBMol.GetBond(m+1,mol.getBondedAtoms(m)[i]+1) is not None and mol.getBondedAtoms(m)[i] not in bridgingatoms:
+                        OBMol.DeleteBond(OBMol.GetBond(m+1,mol.getBondedAtoms(m)[i]+1))
+                        #print('FFopt deleting bond')
+                        deleted_bonds += 1
+                print('FFopt deleted ' +str(deleted_bonds) + ' bonds')
+                # then add back one metal-ligand bond for FF
+                if OBMol.GetAtom(m+1).GetValence() == 0:
+                    for i in mol.getBondedAtoms(m):#getBondedAtomsOct(m,deleted_bonds+len(bridgingatoms)):
+                        if OBMol.GetAtom(m+1).GetValence() < 1 and i not in bridgingatoms:
+                            OBMol.AddBond(m+1,i+1,1)
         # freeze small ligands
         for cat in frozenats:
             constr.AddAtomConstraint(cat+1) # indexing babel
@@ -786,77 +808,79 @@ def rotate_catom_fix_Hs(lig3D,catoms,n,mcoords,core3D):
             bridginggroup = subm
             if list(set(subm).intersection(lig3D.getBondedAtoms(catoms[n])))[0] not in anchoratoms:
                 anchoratoms.append(list(set(subm).intersection(lig3D.getBondedAtoms(catoms[n])))[0])
-    for atom in danglinggroup:
-        confrag3D.addAtom(lig3D.getAtom(atom))
-        confragatomlist.append(atom)
-    if confrag3D.natoms > 1:
-        # terminal connecting atom
-        confrag3Dtmp = mol3D()
-        confrag3Dtmp.copymol3D(confrag3D)
-        if len(anchoratoms) == 1:
-            anchoratom = anchoratoms[0]
-            anchor = lig3D.getAtomCoords(anchoratom)
-            if not checkcolinear(anchor,confrag3D.getAtomCoords(0),confrag3D.getAtomCoords(1)):
+
+    if not len(anchoratoms) == 1:
+        for atom in danglinggroup:
+            confrag3D.addAtom(lig3D.getAtom(atom))
+            confragatomlist.append(atom)
+        if confrag3D.natoms > 1:
+            # terminal connecting atom
+            confrag3Dtmp = mol3D()
+            confrag3Dtmp.copymol3D(confrag3D)
+            if len(anchoratoms) == 1:
+                anchoratom = anchoratoms[0]
+                anchor = lig3D.getAtomCoords(anchoratom)
+                if not checkcolinear(anchor,confrag3D.getAtomCoords(0),confrag3D.getAtomCoords(1)):
+                    refpt = confrag3D.getAtomCoords(0)
+                    u = vecdiff(refpt,anchor)
+                    dtheta = 5
+                    objs = []
+                    objopt = 0
+                    localmaxs = []
+                    thetas = range(0,360,dtheta)
+                    for theta in thetas:
+                        confrag3Dtmp = rotate_around_axis(confrag3Dtmp,refpt,u,dtheta)
+                        auxmol1 = mol3D()
+                        auxmol1.addAtom(confrag3Dtmp.getAtom(0))
+                        for at in confrag3Dtmp.getBondedAtoms(0):
+                            auxmol1.addAtom(confrag3Dtmp.getAtom(at))
+                        auxmol1.addAtom(lig3D.getAtom(anchoratom))
+                        auxmol2 = mol3D()
+                        auxmol2.copymol3D(confrag3Dtmp)
+                        #objs.append(distance(mcoords,auxmol.centersym()))
+                        if auxmol2.natoms > 3:
+                            obj = auxmol2.mindisttopoint(mcoords)
+                        else:
+                            obj = distance(mcoords,auxmol1.centersym())
+                        if obj > objopt:
+                            objopt = obj
+                            thetaopt = theta
+                    #for i,obj in enumerate(objs):
+                        #try:
+                            #if objs[i] > objs[i-1] and objs[i] > objs[i+1]:
+                                #localmaxs.append(thetas[i])
+                        #except IndexError:
+                            #pass
+                ## in future, compare multiple local maxima
+                #if localmaxs == []:
+                    #localmaxs = [0]
+                confrag3D = rotate_around_axis(confrag3D,refpt,u,thetaopt)
+                #confrag3D = rotate_around_axis(confrag3D,refpt,u,localmaxs[0])
+            # non-terminal connecting atom
+            elif len(anchoratoms) == 2:
                 refpt = confrag3D.getAtomCoords(0)
-                u = vecdiff(refpt,anchor)
+                anchorcoords1 = lig3D.getAtomCoords(anchoratoms[0])
+                anchorcoords2 = lig3D.getAtomCoords(anchoratoms[1])
+                u = vecdiff(anchorcoords1,anchorcoords2)
                 dtheta = 5
                 objs = []
-                objopt = 0
                 localmaxs = []
                 thetas = range(0,360,dtheta)
                 for theta in thetas:
                     confrag3Dtmp = rotate_around_axis(confrag3Dtmp,refpt,u,dtheta)
-                    auxmol1 = mol3D()
-                    auxmol1.addAtom(confrag3Dtmp.getAtom(0))
-                    for at in confrag3Dtmp.getBondedAtoms(0):
-                        auxmol1.addAtom(confrag3Dtmp.getAtom(at))
-                    auxmol1.addAtom(lig3D.getAtom(anchoratom))
-                    auxmol2 = mol3D()
-                    auxmol2.copymol3D(confrag3Dtmp)
-                    #objs.append(distance(mcoords,auxmol.centersym()))
-                    if auxmol2.natoms > 3:
-                        obj = auxmol2.mindisttopoint(mcoords)
-                    else:
-                        obj = distance(mcoords,auxmol1.centersym())
-                    if obj > objopt:
-                        objopt = obj
-                        thetaopt = theta
-                #for i,obj in enumerate(objs):
-                    #try:
-                        #if objs[i] > objs[i-1] and objs[i] > objs[i+1]:
-                            #localmaxs.append(thetas[i])
-                    #except IndexError:
-                        #pass
-            ## in future, compare multiple local maxima
-            #if localmaxs == []:
-                #localmaxs = [0]
-            confrag3D = rotate_around_axis(confrag3D,refpt,u,thetaopt)
-            #confrag3D = rotate_around_axis(confrag3D,refpt,u,localmaxs[0])
-        # non-terminal connecting atom
-        elif len(anchoratoms) == 2:
-            refpt = confrag3D.getAtomCoords(0)
-            anchorcoords1 = lig3D.getAtomCoords(anchoratoms[0])
-            anchorcoords2 = lig3D.getAtomCoords(anchoratoms[1])
-            u = vecdiff(anchorcoords1,anchorcoords2)
-            dtheta = 5
-            objs = []
-            localmaxs = []
-            thetas = range(0,360,dtheta)
-            for theta in thetas:
-                confrag3Dtmp = rotate_around_axis(confrag3Dtmp,refpt,u,dtheta)
-                newHcoords = confrag3Dtmp.getAtomCoords(1)
-                objs.append(distance(newHcoords,anchorcoords1)+distance(newHcoords,anchorcoords2)+distance(newHcoords,mcoords))
-            for i,obj in enumerate(objs):
-                try:
-                    if objs[i] > objs[i-1] and objs[i] > objs[i+1]:
-                        localmaxs.append(thetas[i])
-                except IndexError:
-                    pass
-            if localmaxs == []:
-                localmaxs = [0]
-            confrag3D = rotate_around_axis(confrag3D,refpt,u,localmaxs[0])
-        for i,atom in enumerate(confragatomlist):
-            lig3D.getAtom(atom).setcoords(confrag3D.getAtomCoords(i))
+                    newHcoords = confrag3Dtmp.getAtomCoords(1)
+                    objs.append(distance(newHcoords,anchorcoords1)+distance(newHcoords,anchorcoords2)+distance(newHcoords,mcoords))
+                for i,obj in enumerate(objs):
+                    try:
+                        if objs[i] > objs[i-1] and objs[i] > objs[i+1]:
+                            localmaxs.append(thetas[i])
+                    except IndexError:
+                        pass
+                if localmaxs == []:
+                    localmaxs = [0]
+                confrag3D = rotate_around_axis(confrag3D,refpt,u,localmaxs[0])
+            for i,atom in enumerate(confragatomlist):
+                lig3D.getAtom(atom).setcoords(confrag3D.getAtomCoords(i))
     lig3D_aligned = mol3D()
     lig3D_aligned.copymol3D(lig3D)
     return lig3D_aligned
@@ -1091,6 +1115,7 @@ def align_dent2_catom2_coarse(args,lig3D,core3D,catoms,r1,r0,m3D,batoms,corerefc
             objopt = obj
             lig3Dopt = mol3D()
             lig3Dopt.copymol3D(lig3D_tmp)
+
     lig3D = mol3D()
     lig3D.copymol3D(lig3Dopt)
     tmp3D = mol3D()
@@ -1102,6 +1127,7 @@ def align_dent2_catom2_coarse(args,lig3D,core3D,catoms,r1,r0,m3D,batoms,corerefc
     #lig3D = rotate_around_axis(lig3D,r1,urot,theta)
     #lig3Db = rotate_around_axis(lig3Db,r1,urot,-theta)
     # select best
+
     try:
         rm0,rm1 = lig3D.centermass(),lig3Db.centermass()
         theta,ul0 = rotation_params(rm0,r0l,r1l)
@@ -1600,7 +1626,8 @@ def mcomplex(args,ligs,ligoc,licores,globs):
                 cpoint = m3D.getAtom(batoms[0])
                 # attach ligand depending on the denticity
                 # optimize geometry by minimizing steric effects
-                print batoms
+                if args.debug:
+                    print('backbone atoms: ' + str(batoms))
                 if (denticity == 1):
                     lig3D,MLoptbds = align_dent1_lig(args,cpoint,core3D,coreref,ligand,lig3D,catoms,rempi,ligpiatoms,MLb,ANN_flag,ANN_bondl,this_diag,MLbonds,MLoptbds,i)
                 elif (denticity == 2):
@@ -1883,8 +1910,13 @@ def structgen(args,rootdir,ligands,ligoc,globs,sernum):
 
     strfiles = []
 
-    if args.smicat:
+    #for ligs in ligands:
+        #print(ligs)
+        #sad
+
+    if args.smicat: #or True:
         if sum([len(i)>1 for i in args.smicat]) > 0:
+        #if True:
             print('You have specified multidentate SMILES ligand(s).')
             print('We will automatically find suitable conformer(s) for coordination.')
             for n in range(1,int(args.nconfs)+1):
